from buf.validate import validate_pb2 as _validate_pb2
from google.api import annotations_pb2 as _annotations_pb2
from google.api import field_behavior_pb2 as _field_behavior_pb2
from google.api import visibility_pb2 as _visibility_pb2
from google.protobuf import any_pb2 as _any_pb2
from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import empty_pb2 as _empty_pb2
from google.protobuf import field_mask_pb2 as _field_mask_pb2
from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf import wrappers_pb2 as _wrappers_pb2
from protoc_gen_openapiv2.options import annotations_pb2 as _annotations_pb2_1
from scalekit.v1.commons import commons_pb2 as _commons_pb2
from scalekit.v1.options import options_pb2 as _options_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Feature(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    FEATURE_UNSPECIFIED: _ClassVar[Feature]
    UNSPECIFIED: _ClassVar[Feature]
    dir_sync: _ClassVar[Feature]
    sso: _ClassVar[Feature]
FEATURE_UNSPECIFIED: Feature
UNSPECIFIED: Feature
dir_sync: Feature
sso: Feature

class CreateOrganizationRequest(_message.Message):
    __slots__ = ("organization",)
    ORGANIZATION_FIELD_NUMBER: _ClassVar[int]
    organization: CreateOrganization
    def __init__(self, organization: _Optional[_Union[CreateOrganization, _Mapping]] = ...) -> None: ...

class CreateOrganizationResponse(_message.Message):
    __slots__ = ("organization",)
    ORGANIZATION_FIELD_NUMBER: _ClassVar[int]
    organization: Organization
    def __init__(self, organization: _Optional[_Union[Organization, _Mapping]] = ...) -> None: ...

class CreateOrganization(_message.Message):
    __slots__ = ("display_name", "region_code", "external_id", "metadata")
    class MetadataEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    REGION_CODE_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    display_name: str
    region_code: _commons_pb2.RegionCode
    external_id: str
    metadata: _containers.ScalarMap[str, str]
    def __init__(self, display_name: _Optional[str] = ..., region_code: _Optional[_Union[_commons_pb2.RegionCode, str]] = ..., external_id: _Optional[str] = ..., metadata: _Optional[_Mapping[str, str]] = ...) -> None: ...

class Organization(_message.Message):
    __slots__ = ("id", "create_time", "update_time", "display_name", "region_code", "external_id", "metadata", "settings")
    class MetadataEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    ID_FIELD_NUMBER: _ClassVar[int]
    CREATE_TIME_FIELD_NUMBER: _ClassVar[int]
    UPDATE_TIME_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    REGION_CODE_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SETTINGS_FIELD_NUMBER: _ClassVar[int]
    id: str
    create_time: _timestamp_pb2.Timestamp
    update_time: _timestamp_pb2.Timestamp
    display_name: str
    region_code: _commons_pb2.RegionCode
    external_id: str
    metadata: _containers.ScalarMap[str, str]
    settings: OrganizationSettings
    def __init__(self, id: _Optional[str] = ..., create_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., update_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., display_name: _Optional[str] = ..., region_code: _Optional[_Union[_commons_pb2.RegionCode, str]] = ..., external_id: _Optional[str] = ..., metadata: _Optional[_Mapping[str, str]] = ..., settings: _Optional[_Union[OrganizationSettings, _Mapping]] = ...) -> None: ...

class UpdateOrganizationRequest(_message.Message):
    __slots__ = ("id", "external_id", "organization", "update_mask")
    ID_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    ORGANIZATION_FIELD_NUMBER: _ClassVar[int]
    UPDATE_MASK_FIELD_NUMBER: _ClassVar[int]
    id: str
    external_id: str
    organization: UpdateOrganization
    update_mask: _field_mask_pb2.FieldMask
    def __init__(self, id: _Optional[str] = ..., external_id: _Optional[str] = ..., organization: _Optional[_Union[UpdateOrganization, _Mapping]] = ..., update_mask: _Optional[_Union[_field_mask_pb2.FieldMask, _Mapping]] = ...) -> None: ...

class UpdateOrganization(_message.Message):
    __slots__ = ("display_name", "external_id", "metadata")
    class MetadataEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    display_name: str
    external_id: str
    metadata: _containers.ScalarMap[str, str]
    def __init__(self, display_name: _Optional[str] = ..., external_id: _Optional[str] = ..., metadata: _Optional[_Mapping[str, str]] = ...) -> None: ...

class UpdateOrganizationResponse(_message.Message):
    __slots__ = ("organization",)
    ORGANIZATION_FIELD_NUMBER: _ClassVar[int]
    organization: Organization
    def __init__(self, organization: _Optional[_Union[Organization, _Mapping]] = ...) -> None: ...

class GetOrganizationRequest(_message.Message):
    __slots__ = ("id", "external_id")
    ID_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    external_id: str
    def __init__(self, id: _Optional[str] = ..., external_id: _Optional[str] = ...) -> None: ...

class GetOrganizationResponse(_message.Message):
    __slots__ = ("organization",)
    ORGANIZATION_FIELD_NUMBER: _ClassVar[int]
    organization: Organization
    def __init__(self, organization: _Optional[_Union[Organization, _Mapping]] = ...) -> None: ...

class ListOrganizationsRequest(_message.Message):
    __slots__ = ("page_size", "page_token", "external_id")
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    page_size: int
    page_token: str
    external_id: str
    def __init__(self, page_size: _Optional[int] = ..., page_token: _Optional[str] = ..., external_id: _Optional[str] = ...) -> None: ...

class ListOrganizationsResponse(_message.Message):
    __slots__ = ("next_page_token", "total_size", "organizations", "prev_page_token")
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    TOTAL_SIZE_FIELD_NUMBER: _ClassVar[int]
    ORGANIZATIONS_FIELD_NUMBER: _ClassVar[int]
    PREV_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    next_page_token: str
    total_size: int
    organizations: _containers.RepeatedCompositeFieldContainer[Organization]
    prev_page_token: str
    def __init__(self, next_page_token: _Optional[str] = ..., total_size: _Optional[int] = ..., organizations: _Optional[_Iterable[_Union[Organization, _Mapping]]] = ..., prev_page_token: _Optional[str] = ...) -> None: ...

class SearchOrganizationsRequest(_message.Message):
    __slots__ = ("query", "page_size", "page_token")
    QUERY_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    query: str
    page_size: int
    page_token: str
    def __init__(self, query: _Optional[str] = ..., page_size: _Optional[int] = ..., page_token: _Optional[str] = ...) -> None: ...

class SearchOrganizationsResponse(_message.Message):
    __slots__ = ("next_page_token", "total_size", "organizations", "prev_page_token")
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    TOTAL_SIZE_FIELD_NUMBER: _ClassVar[int]
    ORGANIZATIONS_FIELD_NUMBER: _ClassVar[int]
    PREV_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    next_page_token: str
    total_size: int
    organizations: _containers.RepeatedCompositeFieldContainer[Organization]
    prev_page_token: str
    def __init__(self, next_page_token: _Optional[str] = ..., total_size: _Optional[int] = ..., organizations: _Optional[_Iterable[_Union[Organization, _Mapping]]] = ..., prev_page_token: _Optional[str] = ...) -> None: ...

class DeleteOrganizationRequest(_message.Message):
    __slots__ = ("id", "external_id")
    ID_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    external_id: str
    def __init__(self, id: _Optional[str] = ..., external_id: _Optional[str] = ...) -> None: ...

class GeneratePortalLinkRequest(_message.Message):
    __slots__ = ("id", "sso", "directory_sync", "features")
    ID_FIELD_NUMBER: _ClassVar[int]
    SSO_FIELD_NUMBER: _ClassVar[int]
    DIRECTORY_SYNC_FIELD_NUMBER: _ClassVar[int]
    FEATURES_FIELD_NUMBER: _ClassVar[int]
    id: str
    sso: bool
    directory_sync: bool
    features: _containers.RepeatedScalarFieldContainer[Feature]
    def __init__(self, id: _Optional[str] = ..., sso: bool = ..., directory_sync: bool = ..., features: _Optional[_Iterable[_Union[Feature, str]]] = ...) -> None: ...

class GetPortalLinkRequest(_message.Message):
    __slots__ = ("id",)
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class DeletePortalLinkRequest(_message.Message):
    __slots__ = ("id",)
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class DeletePortalLinkByIdRequest(_message.Message):
    __slots__ = ("id", "link_id")
    ID_FIELD_NUMBER: _ClassVar[int]
    LINK_ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    link_id: str
    def __init__(self, id: _Optional[str] = ..., link_id: _Optional[str] = ...) -> None: ...

class Link(_message.Message):
    __slots__ = ("id", "location", "expire_time")
    ID_FIELD_NUMBER: _ClassVar[int]
    LOCATION_FIELD_NUMBER: _ClassVar[int]
    EXPIRE_TIME_FIELD_NUMBER: _ClassVar[int]
    id: str
    location: str
    expire_time: _timestamp_pb2.Timestamp
    def __init__(self, id: _Optional[str] = ..., location: _Optional[str] = ..., expire_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class GeneratePortalLinkResponse(_message.Message):
    __slots__ = ("link",)
    LINK_FIELD_NUMBER: _ClassVar[int]
    link: Link
    def __init__(self, link: _Optional[_Union[Link, _Mapping]] = ...) -> None: ...

class GetPortalLinksResponse(_message.Message):
    __slots__ = ("links",)
    LINKS_FIELD_NUMBER: _ClassVar[int]
    links: _containers.RepeatedCompositeFieldContainer[Link]
    def __init__(self, links: _Optional[_Iterable[_Union[Link, _Mapping]]] = ...) -> None: ...

class UpdateOrganizationSettingsRequest(_message.Message):
    __slots__ = ("id", "settings")
    ID_FIELD_NUMBER: _ClassVar[int]
    SETTINGS_FIELD_NUMBER: _ClassVar[int]
    id: str
    settings: OrganizationSettings
    def __init__(self, id: _Optional[str] = ..., settings: _Optional[_Union[OrganizationSettings, _Mapping]] = ...) -> None: ...

class UpdateOrganizationSessionSettingsRequest(_message.Message):
    __slots__ = ("id", "environment_id", "session_settings")
    ID_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_ID_FIELD_NUMBER: _ClassVar[int]
    SESSION_SETTINGS_FIELD_NUMBER: _ClassVar[int]
    id: str
    environment_id: str
    session_settings: OrganizationSessionSettings
    def __init__(self, id: _Optional[str] = ..., environment_id: _Optional[str] = ..., session_settings: _Optional[_Union[OrganizationSessionSettings, _Mapping]] = ...) -> None: ...

class UpdateOrganizationSessionSettingsResponse(_message.Message):
    __slots__ = ("environment_id", "organization_id", "session_settings")
    ENVIRONMENT_ID_FIELD_NUMBER: _ClassVar[int]
    ORGANIZATION_ID_FIELD_NUMBER: _ClassVar[int]
    SESSION_SETTINGS_FIELD_NUMBER: _ClassVar[int]
    environment_id: str
    organization_id: str
    session_settings: OrganizationSessionSettings
    def __init__(self, environment_id: _Optional[str] = ..., organization_id: _Optional[str] = ..., session_settings: _Optional[_Union[OrganizationSessionSettings, _Mapping]] = ...) -> None: ...

class OrganizationUserManagementSettings(_message.Message):
    __slots__ = ("jit_provisioning_with_sso_enabled", "sync_user_profile_on_signin", "deprecated_placeholder")
    JIT_PROVISIONING_WITH_SSO_ENABLED_FIELD_NUMBER: _ClassVar[int]
    SYNC_USER_PROFILE_ON_SIGNIN_FIELD_NUMBER: _ClassVar[int]
    DEPRECATED_PLACEHOLDER_FIELD_NUMBER: _ClassVar[int]
    jit_provisioning_with_sso_enabled: _wrappers_pb2.BoolValue
    sync_user_profile_on_signin: _wrappers_pb2.BoolValue
    deprecated_placeholder: _struct_pb2.NullValue
    def __init__(self, jit_provisioning_with_sso_enabled: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ..., sync_user_profile_on_signin: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ..., deprecated_placeholder: _Optional[_Union[_struct_pb2.NullValue, str]] = ...) -> None: ...

class OrganizationSessionSettings(_message.Message):
    __slots__ = ("absolute_session_timeout", "session_management_enabled", "idle_session_timeout", "idle_session_enabled")
    ABSOLUTE_SESSION_TIMEOUT_FIELD_NUMBER: _ClassVar[int]
    SESSION_MANAGEMENT_ENABLED_FIELD_NUMBER: _ClassVar[int]
    IDLE_SESSION_TIMEOUT_FIELD_NUMBER: _ClassVar[int]
    IDLE_SESSION_ENABLED_FIELD_NUMBER: _ClassVar[int]
    absolute_session_timeout: _wrappers_pb2.Int32Value
    session_management_enabled: _wrappers_pb2.BoolValue
    idle_session_timeout: _wrappers_pb2.Int32Value
    idle_session_enabled: _wrappers_pb2.BoolValue
    def __init__(self, absolute_session_timeout: _Optional[_Union[_wrappers_pb2.Int32Value, _Mapping]] = ..., session_management_enabled: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ..., idle_session_timeout: _Optional[_Union[_wrappers_pb2.Int32Value, _Mapping]] = ..., idle_session_enabled: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...) -> None: ...

class GetOrganizationSessionSettingsRequest(_message.Message):
    __slots__ = ("id", "environment_id")
    ID_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    environment_id: str
    def __init__(self, id: _Optional[str] = ..., environment_id: _Optional[str] = ...) -> None: ...

class CreateOrganizationSessionSettingsRequest(_message.Message):
    __slots__ = ("id", "environment_id")
    ID_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    environment_id: str
    def __init__(self, id: _Optional[str] = ..., environment_id: _Optional[str] = ...) -> None: ...

class CreateOrganizationSessionSettingsResponse(_message.Message):
    __slots__ = ("environment_id", "organization_id", "session_settings")
    ENVIRONMENT_ID_FIELD_NUMBER: _ClassVar[int]
    ORGANIZATION_ID_FIELD_NUMBER: _ClassVar[int]
    SESSION_SETTINGS_FIELD_NUMBER: _ClassVar[int]
    environment_id: str
    organization_id: str
    session_settings: OrganizationSessionSettings
    def __init__(self, environment_id: _Optional[str] = ..., organization_id: _Optional[str] = ..., session_settings: _Optional[_Union[OrganizationSessionSettings, _Mapping]] = ...) -> None: ...

class GetOrganizationSessionSettingsResponse(_message.Message):
    __slots__ = ("environment_id", "organization_id", "session_settings")
    ENVIRONMENT_ID_FIELD_NUMBER: _ClassVar[int]
    ORGANIZATION_ID_FIELD_NUMBER: _ClassVar[int]
    SESSION_SETTINGS_FIELD_NUMBER: _ClassVar[int]
    environment_id: str
    organization_id: str
    session_settings: OrganizationSessionSettings
    def __init__(self, environment_id: _Optional[str] = ..., organization_id: _Optional[str] = ..., session_settings: _Optional[_Union[OrganizationSessionSettings, _Mapping]] = ...) -> None: ...

class DeleteOrganizationSessionSettingsRequest(_message.Message):
    __slots__ = ("id", "environment_id")
    ID_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    environment_id: str
    def __init__(self, id: _Optional[str] = ..., environment_id: _Optional[str] = ...) -> None: ...

class OrganizationSettings(_message.Message):
    __slots__ = ("features",)
    FEATURES_FIELD_NUMBER: _ClassVar[int]
    features: _containers.RepeatedCompositeFieldContainer[OrganizationSettingsFeature]
    def __init__(self, features: _Optional[_Iterable[_Union[OrganizationSettingsFeature, _Mapping]]] = ...) -> None: ...

class OrganizationSettingsFeature(_message.Message):
    __slots__ = ("name", "enabled")
    NAME_FIELD_NUMBER: _ClassVar[int]
    ENABLED_FIELD_NUMBER: _ClassVar[int]
    name: str
    enabled: bool
    def __init__(self, name: _Optional[str] = ..., enabled: bool = ...) -> None: ...

class UpdateUserManagementSettingsRequest(_message.Message):
    __slots__ = ("organization_id", "settings")
    ORGANIZATION_ID_FIELD_NUMBER: _ClassVar[int]
    SETTINGS_FIELD_NUMBER: _ClassVar[int]
    organization_id: str
    settings: OrganizationUserManagementSettings
    def __init__(self, organization_id: _Optional[str] = ..., settings: _Optional[_Union[OrganizationUserManagementSettings, _Mapping]] = ...) -> None: ...

class UpdateUserManagementSettingsResponse(_message.Message):
    __slots__ = ("settings",)
    SETTINGS_FIELD_NUMBER: _ClassVar[int]
    settings: OrganizationUserManagementSettings
    def __init__(self, settings: _Optional[_Union[OrganizationUserManagementSettings, _Mapping]] = ...) -> None: ...

class GetOrganizationUserManagementSettingsRequest(_message.Message):
    __slots__ = ("organization_id",)
    ORGANIZATION_ID_FIELD_NUMBER: _ClassVar[int]
    organization_id: str
    def __init__(self, organization_id: _Optional[str] = ...) -> None: ...

class GetOrganizationUserManagementSettingsResponse(_message.Message):
    __slots__ = ("settings",)
    SETTINGS_FIELD_NUMBER: _ClassVar[int]
    settings: OrganizationUserManagementSettings
    def __init__(self, settings: _Optional[_Union[OrganizationUserManagementSettings, _Mapping]] = ...) -> None: ...



import json
from math import floor
from typing import Any, Optional, Dict
from urllib.parse import urlencode

import jwt
import hmac
import hashlib
import base64
from datetime import datetime, timedelta, timezone
from scalekit.core import CoreClient
from scalekit.domain import DomainClient
from scalekit.connection import ConnectionClient
from scalekit.m2m_client import M2MClient
from scalekit.organization import OrganizationClient
from scalekit.directory import DirectoryClient
from scalekit.users import UserClient
from scalekit.role import RoleClient
from scalekit.connected_accounts import ConnectedAccountsClient
from scalekit.tools import ToolsClient
from scalekit.connect import ConnectClient
from scalekit.passwordless import PasswordlessClient
from scalekit.mcp import McpClient
from scalekit.common.scalekit import (
    AuthorizationUrlOptions,
    CodeAuthenticationOptions,
    GrantType,
    IdpInitiatedLoginClaims,
    LogoutUrlOptions,
    TokenValidationOptions,
)
from scalekit.constants.user import id_token_claim_to_user_map
from scalekit.common.exceptions import (WebhookVerificationError,
                                        ScalekitValidateTokenFailureException)

AUTHORIZE_ENDPOINT = "oauth/authorize"
LOGOUT_ENDPOINT = "oidc/logout" 
webhook_tolerance_in_seconds = timedelta(minutes=5)
webhook_signature_version = "v1"


class ScalekitClient:
    """ Class definition for scalekit client """

    def __init__(self, env_url: str, client_id: str, client_secret: str):
        """
        Initializer for Scalekit base class

        :param env_url        : Environment URL
        :type                 : ``` str ```
        :param client_id      : Client ID
        :type                 : ``` str ```
        :param client_secret  : Client Secret
        :type                 : ``` str ```

        :returns:
            None
        """
        try:
            self.core_client = CoreClient(
                env_url=env_url, client_id=client_id, client_secret=client_secret)
            self.domain = DomainClient(self.core_client)
            self.connection = ConnectionClient(self.core_client)
            self.organization = OrganizationClient(self.core_client)
            self.directory = DirectoryClient(self.core_client)
            self.m2m_client = M2MClient(self.core_client)
            self.users = UserClient(self.core_client)
            self.roles = RoleClient(self.core_client)
            self.connected_accounts = ConnectedAccountsClient(self.core_client)
            self.tools = ToolsClient(self.core_client)
            self.mcp = McpClient(self.core_client)
            self.connect = ConnectClient(self.tools, self.connected_accounts, self.mcp)
            self.passwordless = PasswordlessClient(self.core_client)
        except Exception as exp:
            raise exp

    def get_authorization_url(
        self, redirect_uri: str, options: AuthorizationUrlOptions | None
    ):
        """
        Method to get authorization URL

        :param redirect_uri   : Redirect URI for SAML SSO
        :type                 : ``` str ```
        :param options        : Auth URL options object
        :type                 : ``` obj ```

        :returns:
            Authorization URL
        """
        try:
            scopes = (
                options.scopes if options.scopes else ["openid", "profile", "email"]
            )
            url_params_dict = {
                "response_type": "code",
                "client_id": self.core_client.client_id,
                "redirect_uri": redirect_uri,
                "scope": " ".join(scopes),
                "state": options.state,
                "nonce": options.nonce,
                "login_hint": options.login_hint,
                "domain_hint": options.domain_hint,
                "connection_id": options.connection_id,
                "organization_id": options.organization_id,
                "provider": options.provider,
                "prompt": options.prompt,  
            }

            valid_auth_params = {k: v for k, v in url_params_dict.items() if v}
            query_string = urlencode(valid_auth_params)

            return f"{self.core_client.env_url}/{AUTHORIZE_ENDPOINT}?{query_string}"
        except Exception as exp:
            raise exp

    def authenticate_with_code(
        self, code, redirect_uri, options: CodeAuthenticationOptions
    ):
        """
        Method to authenticate with code options

        :param code           : authorization_code
        :type                 : ``` str ```
        :param redirect_uri   : Redirect URI
        :type                 : ``` str ```
        :param options        : CodeAuthenticationOptions Object
        :type                 : ``` obj ```

        :returns:
            dict with user, id token, optionally access token & refresh token
        """
        try:
            response = self.core_client.authenticate(
                json.dumps(
                    {
                        "code": code,
                        "redirect_uri": redirect_uri,
                        "grant_type": GrantType.AuthorizationCode.value,
                        "client_id": self.core_client.client_id,
                        "client_secret": self.core_client.client_secret,
                        "code_verifier": options.code_verifier,
                    }
                )
            )
            response = json.loads(response.content)
            id_token = response["id_token"]
            access_token = response["access_token"]
            refresh_token = response.get("refresh_token")
            # Validate id_token
            claims = self.__validate_token(id_token, options=None)
            user = {}
            amr_claims = claims.get('amr', [])
            connection_id = amr_claims[0] if amr_claims else None
            organization_id = claims.get('oid')
            for k, v in claims.items():
                if id_token_claim_to_user_map.get(k, None):
                    user[id_token_claim_to_user_map[k]] = v

            return {
                "user": user,
                "id_token": id_token,
                "access_token": access_token,
                "refresh_token": refresh_token,
                "connection_id": connection_id,
                "organization_id": organization_id
            }

        except jwt.exceptions.InvalidTokenError as exp:
            raise ScalekitValidateTokenFailureException(exp)
        except Exception as exp:
            raise exp

    def validate_access_token(self, token: str, options: Optional[TokenValidationOptions] = None, audience = None) -> bool:
        """
        Method to validate access token

        :param token : access token
        :type        : ``` str ```
        :param options : Optional validation options for issuer, audience, and scopes
        :type        : ``` TokenValidationOptions ```
        :param audience : audience for validation (deprecated, use options.audience instead)
        :type        : ``` str ```

        :returns:
            bool
        """
        try:
            self.validate_token(token, options=options, audience=audience)
            return True
        except Exception:
            return False

    def generate_client_token(self, client_id: str, client_secret: str) -> str:
        """
        Method to generate access token

        :param client_id        : Client Id for access token
        :type                           : ``` str ```
        :param client_secret : Client Secret for access token
        :type                            : ``` str ```
        :returns:
            access token
        """
        try:
            response = self.core_client.authenticate(
                json.dumps(
                    {
                        "grant_type": GrantType.ClientCredentials.value,
                        "client_id": client_id,
                        "client_secret": client_secret,
                    }
                )
            )
            response = json.loads(response.content)
            return response
        except Exception as exp:
            raise exp

    def validate_access_token_and_get_claims(self, token: str, options: Optional[TokenValidationOptions] = None, audience = None) -> Dict[str, Any]:
        """
        Method to validate access token and get claims

        :param token : access token
        :type        : ``` str ```
        :param options : Optional validation options for issuer, audience, and scopes
        :type        : ``` TokenValidationOptions ```
        :param audience : audience for validation (deprecated, use options.audience instead)
        :type        : ``` str ```

        :returns:
            claims
        """
        
        try:
            claims = self.validate_token(token, options=options, audience=audience)
            return claims
        except jwt.exceptions.InvalidTokenError as exp:
            raise ScalekitValidateTokenFailureException(exp)
        except Exception as exp:
            raise exp

    def get_idp_initiated_login_claims(self, idp_initiated_login_token: str, options: Optional[TokenValidationOptions] = None, audience = None) -> IdpInitiatedLoginClaims:
        """
        Method to get IDP initiated login claims

        :param idp_initiated_login_token : IDP initiated login token
        :type        : ``` str ```
        :param options : Optional validation options for issuer and audience
        :type        : ``` TokenValidationOptions ```
        :param audience : audience for validation (deprecated, use options.audience instead)
        :type        : ``` str ```

        :returns:
            ``` IdpInitiatedLoginClaims ```
        """
        try:
            claims = self.validate_token(idp_initiated_login_token, options=options, audience=audience)
            return claims
        except jwt.exceptions.InvalidTokenError as exp:
            raise ScalekitValidateTokenFailureException(exp)
        except Exception as exp:
            raise exp

    def validate_token(
        self, token: str, options: Optional[TokenValidationOptions] = None, audience: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        Method to validate token

        :param token : token
        :type        : ``` str ```
        :param options : validation options for issuer, audience, and scopes
        :type        : ``` TokenValidationOptions ```
        :param audience : audience for validation
        :type        : ``` str ```

        :returns:
            payload
        """
        # Convert TokenValidationOptions to jwt decode options if provided
        jwt_options = {}
        if options:
            if options.issuer:
                jwt_options["issuer"] = options.issuer
                jwt_options["verify_iss"] = True
            if options.audience:
                jwt_options["audience"] = options.audience
                jwt_options["verify_aud"] = True
            elif audience is not None:
                jwt_options["audience"] = [audience]
                jwt_options["verify_aud"] = True
            else:
                jwt_options["verify_aud"] = False
        
        self.core_client.get_jwks()
        kid = jwt.get_unverified_header(token)["kid"]
        key = self.core_client.keys[kid]

        decode_opts = {
            "verify_iss": jwt_options.pop("verify_iss", False),
            "verify_aud": jwt_options.pop("verify_aud", False)
        }

        payload = jwt.decode(
            token,
            key=key,
            algorithms=["RS256"],
            options=decode_opts,
            **jwt_options,
        )
        
        # Validate scopes if required
        if options and options.required_scopes:
            self.verify_scopes(token, options.required_scopes)
        
        return payload



    def verify_scopes(self, token: str, required_scopes: list[str]) -> bool:
        """
        Verify that the token contains the required scopes

        :param token : The token to verify
        :type        : ``` str ```
        :param required_scopes : The scopes that must be present in the token
        :type        : ``` list[str] ```

        :returns:
            bool: Returns True if all required scopes are present
        :raises:
            Error: If required scopes are missing, with details about which scopes are missing
        """
        payload = jwt.decode(token, options={"verify_signature": False})
        scopes = self.__extract_scopes_from_payload(payload)
        
        missing_scopes = [scope for scope in required_scopes if scope not in scopes]
        
        if missing_scopes:
            raise ScalekitValidateTokenFailureException(f"Token missing required scopes: {', '.join(missing_scopes)}")
        
        return True

    def __extract_scopes_from_payload(self, payload: Dict[str, Any]) -> list[str]:
        """
        Extract scopes from token payload

        :param payload : The token payload
        :type        : ``` Dict[str, Any] ```

        :returns:
            list[str]: Array of scopes found in the token
        """
        scopes = payload.get("scopes", [])
        return [scope for scope in scopes if scope and scope.strip()]

    def verify_webhook_payload(self, secret: str, headers: Dict[str, str], payload: [str | bytes]) -> bool:
        """
        Method to verify webhook payload

        :param secret      :     Secret for webhook verification
        :type              :     ``` str ```
        :param headers     :     Webhook request headers
        :type              :     ``` dict[str, str] ```
        :param payload     :     Webhook payload in str or bytes
        :type              :     ``` str | bytes ```

        :returns:
            bool
        """
        payload = payload if isinstance(payload, str) else payload.decode()
        webhook_id = headers.get("webhook-id")
        webhook_timestamp = headers.get("webhook-timestamp")
        webhook_signature = headers.get("webhook-signature")

        if not all([webhook_id, webhook_timestamp, webhook_signature]):
            raise WebhookVerificationError("Missing required headers")

        secret_parts = secret.split("_")
        if len(secret_parts) < 2:
            raise WebhookVerificationError("Invalid secret")

        try:
            secret_bytes = base64.b64decode(secret_parts[1])
        except Exception as exp:
            raise exp

        try:
            timestamp = self.__verify_timestamp(webhook_timestamp)
        except Exception as exp:
            raise exp

        timestamp_str = str(floor(timestamp.replace(tzinfo=timezone.utc).timestamp()))
        data = f"{webhook_id}.{timestamp_str}.{payload}"
        computed_signature = base64.b64decode(self.__compute_signature(secret_bytes, data).split(',')[1])

        received_signatures = webhook_signature.split(" ")
        for versioned_signature in received_signatures:
            signature_parts = versioned_signature.split(",")
            if len(signature_parts) < 2:
                continue

            version = signature_parts[0]
            signature = base64.b64decode(signature_parts[1])

            if version != webhook_signature_version:
                continue

            if hmac.compare_digest(signature, computed_signature):
                return True

        raise WebhookVerificationError("Invalid signature")

    @staticmethod
    def __verify_timestamp(timestamp_str: str):
        """
        Method to verify time stamp

        :param timestamp_str   :     Timestamp for verification
        :type                  :     ``` str ```

        :returns:
            None
        """
        now = datetime.now(tz=timezone.utc)
        try:
            timestamp = datetime.fromtimestamp(float(timestamp_str), tz=timezone.utc)
        except Exception:
            raise WebhookVerificationError("Invalid Signature Headers")

        if timestamp < (now - webhook_tolerance_in_seconds):
            raise WebhookVerificationError("Message timestamp too old")

        if timestamp > (now + webhook_tolerance_in_seconds):
            raise WebhookVerificationError("Message timestamp too new")

        return timestamp

    @staticmethod
    def __compute_signature(secret: bytes, data: str) -> str:
        """
        Method to compute signature

        :param secret   :     secret for signature
        :type           :     ``` bytes ```
        :param data     :     data for signature
        :type           :     ``` str ```

        :returns:
            None
        """
        signature = hmac.new(secret, data.encode(), hashlib.sha256).digest()
        return f"v1, {base64.b64encode(signature).decode('utf-8')}"

    def refresh_access_token(self, refresh_token: str):
        """
        Method to refresh access token using refresh token

        :param refresh_token  : Refresh token to get new access token
        :type                 : ``` str ```

        :returns:
            dict with access token & refresh token
        """
        try:
            response = self.core_client.authenticate(
                json.dumps(
                    {
                        "refresh_token": refresh_token,
                        "grant_type": GrantType.RefreshToken.value,
                        "client_id": self.core_client.client_id,
                        "client_secret": self.core_client.client_secret,
                    }
                )
            )
            response = json.loads(response.content)
            return {
                "access_token": response["access_token"],
                "refresh_token": response["refresh_token"]
            }

        except Exception as exp:
            raise exp

    def get_logout_url(self, options: LogoutUrlOptions) -> str:
        """
        Method to get logout URL

        :param options        : Logout URL options object
        :type                 : ``` LogoutUrlOptions ```

        :returns:
            str: The logout URL
        """
        try:
            url_params_dict = {
                "id_token_hint": options.id_token_hint,
                "post_logout_redirect_uri": options.post_logout_redirect_uri,
                "state": options.state
            }

            valid_params = {k: v for k, v in url_params_dict.items() if v is not None}
            query_string = urlencode(valid_params)

            return f"{self.core_client.env_url}/{LOGOUT_ENDPOINT}?{query_string}"
        except Exception as exp:
            raise exp



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: scalekit/v1/tools/tools.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.api import visibility_pb2 as google_dot_api_dot_visibility__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from protoc_gen_openapiv2.options import annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2
from scalekit.v1.options import options_pb2 as scalekit_dot_v1_dot_options_dot_options__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dscalekit/v1/tools/tools.proto\x12\x11scalekit.v1.tools\x1a\x1b\x62uf/validate/validate.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x1bgoogle/api/visibility.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a.protoc-gen-openapiv2/options/annotations.proto\x1a!scalekit/v1/options/options.proto\"\x88\x01\n\x11\x43reateToolRequest\x12s\n\x04tool\x18\x01 \x01(\x0b\x32\x17.scalekit.v1.tools.ToolBF\x92\x41=2;Tool details including name, schema version, and definition\xbaH\x03\xc8\x01\x01R\x04tool\"c\n\x12\x43reateToolResponse\x12M\n\x04tool\x18\x01 \x01(\x0b\x32\x17.scalekit.v1.tools.ToolB \x92\x41\x1d\x32\x1b\x44\x65tails of the created toolR\x04tool\"\xf0\x05\n\x04Tool\x12\x38\n\x02id\x18\x01 \x01(\tB(\x92\x41\"2\x15Unique ID of the toolJ\t\"tol_123\"\xe0\x41\x03R\x02id\x12I\n\x08provider\x18\x02 \x01(\tB-\x92\x41\'2\x1bProvider name (e.g. GOOGLE)J\x08\"GOOGLE\"\xe0\x41\x03R\x08provider\x12\x87\x01\n\ndefinition\x18\x03 \x01(\x0b\x32\x17.google.protobuf.StructBN\x92\x41\x45\x32$Tool definition in structured formatJ\x1d{\"input\": {\"type\": \"object\"}}\xbaH\x03\xc8\x01\x01R\ndefinition\x12s\n\x08metadata\x18\x04 \x01(\x0b\x32\x17.google.protobuf.StructB>\x92\x41;2\"Additional metadata about the toolJ\x15{\"category\": \"email\"}R\x08metadata\x12X\n\x04tags\x18\x05 \x03(\tBD\x92\x41\x41\x32$Tags for categorization or filteringJ\x19[\"notification\", \"email\"]R\x04tags\x12\x83\x01\n\nis_default\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.BoolValueBH\x92\x41\x42\x32:Marks this tool as the default version for the combinationJ\x04true\xe0\x41\x03R\tisDefault\x12\x83\x01\n\nupdated_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampBH\x92\x41\x42\x32(Timestamp when the tool was last updatedJ\x16\"2023-10-01T12:00:00Z\"\xe0\x41\x03R\tupdatedAt\"\xd3\x01\n\nScopedTool\x12+\n\x04tool\x18\x01 \x01(\x0b\x32\x17.scalekit.v1.tools.ToolR\x04tool\x12\x1e\n\nidentifier\x18\x02 \x01(\tR\nidentifier\x12x\n\x14\x63onnected_account_id\x18\x03 \x01(\tBF\x92\x41<20ID of the connected account for this scoped toolJ\x08\"ca_123\"\xbaH\x04r\x02\x18\x64R\x12\x63onnectedAccountId\"\xfe\x01\n\x10ListToolsRequest\x12[\n\x06\x66ilter\x18\x01 \x01(\x0b\x32\x19.scalekit.v1.tools.FilterB(\x92\x41%2#Filter parameters for listing toolsR\x06\x66ilter\x12\x1b\n\tpage_size\x18\x02 \x01(\rR\x08pageSize\x12p\n\npage_token\x18\x03 \x01(\tBQ\x92\x41N2.Token from a previous response for pagination.J\x1c\x65yJwYWdlIjoyLCJsaW1pdCI6MzB9R\tpageToken\"\x80\x03\n\x06\x46ilter\x12t\n\x07summary\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.BoolValueB>\x92\x41;23Return only tool names instead of full tool detailsJ\x04trueR\x07summary\x12I\n\x08provider\x18\x02 \x01(\tB-\x92\x41#2\x17\x46ilter by tool providerJ\x08\"GOOGLE\"\xbaH\x04r\x02\x18\x32R\x08provider\x12j\n\nidentifier\x18\x03 \x01(\tBJ\x92\x41@2&Filter by connected account identifierJ\x16\"app_google_workspace\"\xbaH\x04r\x02\x18\x64R\nidentifier\x12I\n\ttool_name\x18\x04 \x03(\tB,\x92\x41)2\x13\x46ilter by tool nameJ\x12\"gmail_send_email\"R\x08toolName\"\x9b\x04\n\x11ListToolsResponse\x12v\n\x0fnext_page_token\x18\x01 \x01(\tBN\x92\x41K2)Token for fetching the next page of toolsJ\x1e\"eyJwYWdlIjozLCJsaW1pdCI6MzB9\"R\rnextPageToken\x12Q\n\ntotal_size\x18\x02 \x01(\rB2\x92\x41/2(Total number of tools matching the queryJ\x03\x31\x30\x34R\ttotalSize\x12z\n\x0fprev_page_token\x18\x03 \x01(\tBR\x92\x41O2-Token for fetching the previous page of toolsJ\x1e\"eyJwYWdlIjoxLCJsaW1pdCI6MzB9\"R\rprevPageToken\x12^\n\ntool_names\x18\x04 \x03(\tB?\x92\x41<2$List of tool names (if summary=true)J\x14[\"gmail_send_email\"]R\ttoolNames\x12_\n\x05tools\x18\x05 \x03(\x0b\x32\x17.scalekit.v1.tools.ToolB0\x92\x41-2+List of tools (if summary=false or omitted)R\x05tools\"\xc9\x07\n\x12\x45xecuteToolRequest\x12Q\n\ttool_name\x18\x01 \x01(\tB4\x92\x41+2\x1bName of the tool to executeJ\x0c\"send_email\"\xbaH\x03\xc8\x01\x01R\x08toolName\x12\x7f\n\nidentifier\x18\x02 \x01(\tBZ\x92\x41W2=Identifier of the connected account to use for tool executionJ\x16\"app_google_workspace\"H\x00R\nidentifier\x88\x01\x01\x12\xb0\x01\n\x06params\x18\x03 \x01(\x0b\x32\x17.google.protobuf.StructB\x7f\x92\x41|23Free-flowing JSON parameters for the tool executionJE{\"to\": \"user@example.com\", \"subject\": \"Hello\", \"body\": \"Hello World\"}R\x06params\x12\x82\x01\n\x14\x63onnected_account_id\x18\x04 \x01(\tBK\x92\x41\x41\x32\x35ID of the connected account to use for tool executionJ\x08\"ca_123\"\xbaH\x04r\x02\x18\x64H\x01R\x12\x63onnectedAccountId\x88\x01\x01\x12z\n\tconnector\x18\x05 \x01(\tBW\x92\x41M27Name of the connected account to use for tool executionJ\x12\"Google Workspace\"\xbaH\x04r\x02\x18\x64H\x02R\tconnector\x88\x01\x01\x12u\n\x0forganization_id\x18\x06 \x01(\tBG\x92\x41=20ID of the organization to use for tool executionJ\t\"org_123\"\xbaH\x04r\x02\x18\x64H\x03R\x0eorganizationId\x88\x01\x01\x12^\n\x07user_id\x18\x07 \x01(\tB@\x92\x41\x36\x32(ID of the user to use for tool executionJ\n\"user_123\"\xbaH\x04r\x02\x18\x64H\x04R\x06userId\x88\x01\x01\x42\r\n\x0b_identifierB\x17\n\x15_connected_account_idB\x0c\n\n_connectorB\x12\n\x10_organization_idB\n\n\x08_user_id\"\xa3\x02\n\x13\x45xecuteToolResponse\x12\xac\x01\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32\x17.google.protobuf.StructB\x7f\x92\x41|23Free-flowing JSON parameters for the tool executionJE{\"to\": \"user@example.com\", \"subject\": \"Hello\", \"body\": \"Hello World\"}R\x04\x64\x61ta\x12]\n\x0c\x65xecution_id\x18\x02 \x01(\tB:\x92\x41\x37\x32(Unique identifier for the tool executionJ\x0b\"123456789\"R\x0b\x65xecutionId\"\xad\x02\n\x15SetToolDefaultRequest\x12\x61\n\x04name\x18\x01 \x01(\tBM\x92\x41/2\x19\x46ully qualified tool nameJ\x12\"gmail_send_email\"\xbaH\x18r\x13\x10\x01\x18\x64\x32\r^[a-z0-9_].*$\xc8\x01\x01R\x04name\x12W\n\x0eschema_version\x18\x02 \x01(\tB0\x92\x41!2\x1aSchema version of the toolJ\x03\"1\"\xbaH\tr\x04\x10\x01\x18\n\xc8\x01\x01R\rschemaVersion\x12X\n\x0ctool_version\x18\x03 \x01(\tB5\x92\x41&2\x1fTool version to mark as defaultJ\x03\"1\"\xbaH\tr\x04\x10\x01\x18\n\xc8\x01\x01R\x0btoolVersion\"j\n\x16SetToolDefaultResponse\x12P\n\x04tool\x18\x01 \x01(\x0b\x32\x17.scalekit.v1.tools.ToolB#\x92\x41 2\x1eThe tool now marked as defaultR\x04tool\"^\n\x11UpdateToolRequest\x12I\n\x04tool\x18\x01 \x01(\x0b\x32\x17.scalekit.v1.tools.ToolB\x1c\x92\x41\x13\x32\x11Updated tool data\xbaH\x03\xc8\x01\x01R\x04tool\"X\n\x12UpdateToolResponse\x12\x42\n\x04tool\x18\x01 \x01(\x0b\x32\x17.scalekit.v1.tools.ToolB\x15\x92\x41\x12\x32\x10The updated toolR\x04tool\"0\n\x11\x44\x65leteToolRequest\x12\x1b\n\x02id\x18\x01 \x01(\tB\x0b\xbaH\x08r\x06:\x04tol_R\x02id\"\x91\x03\n\x16ListScopedToolsRequest\x12z\n\nidentifier\x18\x01 \x01(\tBZ\x92\x41M23Identifier of the connected account to filter toolsJ\x16\"app_google_workspace\"\xbaH\x07r\x02\x18\x64\xc8\x01\x01R\nidentifier\x12l\n\x06\x66ilter\x18\x02 \x01(\x0b\x32#.scalekit.v1.tools.ScopedToolFilterB/\x92\x41,2*Filter parameters for listing scoped toolsR\x06\x66ilter\x12\x1b\n\tpage_size\x18\x03 \x01(\rR\x08pageSize\x12p\n\npage_token\x18\x04 \x01(\tBQ\x92\x41N2.Token from a previous response for pagination.J\x1c\x65yJwYWdlIjoyLCJsaW1pdCI6MzB9R\tpageToken\"\x95\x03\n\x17ListScopedToolsResponse\x12v\n\x0fnext_page_token\x18\x01 \x01(\tBN\x92\x41K2)Token for fetching the next page of toolsJ\x1e\"eyJwYWdlIjozLCJsaW1pdCI6MzB9\"R\rnextPageToken\x12Q\n\ntotal_size\x18\x02 \x01(\rB2\x92\x41/2(Total number of tools matching the queryJ\x03\x31\x30\x34R\ttotalSize\x12z\n\x0fprev_page_token\x18\x03 \x01(\tBR\x92\x41O2-Token for fetching the previous page of toolsJ\x1e\"eyJwYWdlIjoxLCJsaW1pdCI6MzB9\"R\rprevPageToken\x12\x33\n\x05tools\x18\x05 \x03(\x0b\x32\x1d.scalekit.v1.tools.ScopedToolR\x05tools\"z\n\x10ScopedToolFilter\x12\x1c\n\tproviders\x18\x01 \x03(\tR\tproviders\x12\x1d\n\ntool_names\x18\x02 \x03(\tR\ttoolNames\x12)\n\x10\x63onnection_names\x18\x03 \x03(\tR\x0f\x63onnectionNames2\xaa\r\n\x0bToolService\x12\xf2\x01\n\nCreateTool\x12$.scalekit.v1.tools.CreateToolRequest\x1a%.scalekit.v1.tools.CreateToolResponse\"\x96\x01\x92\x41\x63\n\x05Tools\x12\x11\x43reate a new tool\x1aGRegister a new tool with its schema, metadata, and other configuration.\x82\xb5\x18\x02\x18\x04\xfa\xd2\xe4\x93\x02\t\x12\x07PREVIEW\x82\xd3\xe4\x93\x02\x15\"\r/api/v1/tools:\x04tool\x12\xfa\x01\n\tListTools\x12#.scalekit.v1.tools.ListToolsRequest\x1a$.scalekit.v1.tools.ListToolsResponse\"\xa1\x01\x92\x41t\n\x05Tools\x12\nList tools\x1a_Fetch tools by provider, identifier, and connector. If `summary=true`, returns only tool names.\x82\xb5\x18\x02\x18t\xfa\xd2\xe4\x93\x02\t\x12\x07PREVIEW\x82\xd3\xe4\x93\x02\x0f\x12\r/api/v1/tools\x12\x9b\x01\n\x0fListScopedTools\x12).scalekit.v1.tools.ListScopedToolsRequest\x1a*.scalekit.v1.tools.ListScopedToolsResponse\"1\x82\xb5\x18\x02\x18\x04\xfa\xd2\xe4\x93\x02\t\x12\x07PREVIEW\x82\xd3\xe4\x93\x02\x16\x12\x14/api/v1/tools/scoped\x12\xac\x02\n\x0eSetToolDefault\x12(.scalekit.v1.tools.SetToolDefaultRequest\x1a).scalekit.v1.tools.SetToolDefaultResponse\"\xc4\x01\x92\x41\x87\x01\n\x05Tools\x12#Marks a tool as the default version\x1aYSet a specific tool (by name, schema version, tool version, and provider) as the default.\x82\xb5\x18\x02\x18\x04\xfa\xd2\xe4\x93\x02\t\x12\x07PREVIEW\x82\xd3\xe4\x93\x02\x1e\"\x19/api/v1/tools:set_default:\x01*\x12\xd8\x01\n\nUpdateTool\x12$.scalekit.v1.tools.UpdateToolRequest\x1a%.scalekit.v1.tools.UpdateToolResponse\"}\x92\x41J\n\x05Tools\x12\rUpdate a tool\x1a\x32Update an existing tool by provider and tool name.\x82\xb5\x18\x02\x18\x04\xfa\xd2\xe4\x93\x02\t\x12\x07PREVIEW\x82\xd3\xe4\x93\x02\x15\x1a\r/api/v1/tools:\x04tool\x12\xd9\x01\n\nDeleteTool\x12$.scalekit.v1.tools.DeleteToolRequest\x1a\x16.google.protobuf.Empty\"\x8c\x01\x92\x41Z\n\x05Tools\x12\rDelete a tool\x1a\x42\x44\x65lete a tool by name, schema version, tool version, and provider.\x82\xb5\x18\x02\x18\x04\xfa\xd2\xe4\x93\x02\t\x12\x07PREVIEW\x82\xd3\xe4\x93\x02\x14*\x12/api/v1/tools/{id}\x12\xa4\x02\n\x0b\x45xecuteTool\x12%.scalekit.v1.tools.ExecuteToolRequest\x1a&.scalekit.v1.tools.ExecuteToolResponse\"\xc5\x01\x92\x41\x8d\x01\n\rAppConnectors\x12(Execute a tool using a connected account\x1aRExecute a tool using the specified connected account with the provided parameters.\x82\xb5\x18\x02\x18t\xfa\xd2\xe4\x93\x02\t\x12\x07PREVIEW\x82\xd3\xe4\x93\x02\x19\"\x14/api/v1/execute_tool:\x01*B1Z/github.com/scalekit-inc/scalekit/pkg/grpc/toolsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scalekit.v1.tools.tools_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z/github.com/scalekit-inc/scalekit/pkg/grpc/tools'
  _globals['_CREATETOOLREQUEST'].fields_by_name['tool']._loaded_options = None
  _globals['_CREATETOOLREQUEST'].fields_by_name['tool']._serialized_options = b'\222A=2;Tool details including name, schema version, and definition\272H\003\310\001\001'
  _globals['_CREATETOOLRESPONSE'].fields_by_name['tool']._loaded_options = None
  _globals['_CREATETOOLRESPONSE'].fields_by_name['tool']._serialized_options = b'\222A\0352\033Details of the created tool'
  _globals['_TOOL'].fields_by_name['id']._loaded_options = None
  _globals['_TOOL'].fields_by_name['id']._serialized_options = b'\222A\"2\025Unique ID of the toolJ\t\"tol_123\"\340A\003'
  _globals['_TOOL'].fields_by_name['provider']._loaded_options = None
  _globals['_TOOL'].fields_by_name['provider']._serialized_options = b'\222A\'2\033Provider name (e.g. GOOGLE)J\010\"GOOGLE\"\340A\003'
  _globals['_TOOL'].fields_by_name['definition']._loaded_options = None
  _globals['_TOOL'].fields_by_name['definition']._serialized_options = b'\222AE2$Tool definition in structured formatJ\035{\"input\": {\"type\": \"object\"}}\272H\003\310\001\001'
  _globals['_TOOL'].fields_by_name['metadata']._loaded_options = None
  _globals['_TOOL'].fields_by_name['metadata']._serialized_options = b'\222A;2\"Additional metadata about the toolJ\025{\"category\": \"email\"}'
  _globals['_TOOL'].fields_by_name['tags']._loaded_options = None
  _globals['_TOOL'].fields_by_name['tags']._serialized_options = b'\222AA2$Tags for categorization or filteringJ\031[\"notification\", \"email\"]'
  _globals['_TOOL'].fields_by_name['is_default']._loaded_options = None
  _globals['_TOOL'].fields_by_name['is_default']._serialized_options = b'\222AB2:Marks this tool as the default version for the combinationJ\004true\340A\003'
  _globals['_TOOL'].fields_by_name['updated_at']._loaded_options = None
  _globals['_TOOL'].fields_by_name['updated_at']._serialized_options = b'\222AB2(Timestamp when the tool was last updatedJ\026\"2023-10-01T12:00:00Z\"\340A\003'
  _globals['_SCOPEDTOOL'].fields_by_name['connected_account_id']._loaded_options = None
  _globals['_SCOPEDTOOL'].fields_by_name['connected_account_id']._serialized_options = b'\222A<20ID of the connected account for this scoped toolJ\010\"ca_123\"\272H\004r\002\030d'
  _globals['_LISTTOOLSREQUEST'].fields_by_name['filter']._loaded_options = None
  _globals['_LISTTOOLSREQUEST'].fields_by_name['filter']._serialized_options = b'\222A%2#Filter parameters for listing tools'
  _globals['_LISTTOOLSREQUEST'].fields_by_name['page_token']._loaded_options = None
  _globals['_LISTTOOLSREQUEST'].fields_by_name['page_token']._serialized_options = b'\222AN2.Token from a previous response for pagination.J\034eyJwYWdlIjoyLCJsaW1pdCI6MzB9'
  _globals['_FILTER'].fields_by_name['summary']._loaded_options = None
  _globals['_FILTER'].fields_by_name['summary']._serialized_options = b'\222A;23Return only tool names instead of full tool detailsJ\004true'
  _globals['_FILTER'].fields_by_name['provider']._loaded_options = None
  _globals['_FILTER'].fields_by_name['provider']._serialized_options = b'\222A#2\027Filter by tool providerJ\010\"GOOGLE\"\272H\004r\002\0302'
  _globals['_FILTER'].fields_by_name['identifier']._loaded_options = None
  _globals['_FILTER'].fields_by_name['identifier']._serialized_options = b'\222A@2&Filter by connected account identifierJ\026\"app_google_workspace\"\272H\004r\002\030d'
  _globals['_FILTER'].fields_by_name['tool_name']._loaded_options = None
  _globals['_FILTER'].fields_by_name['tool_name']._serialized_options = b'\222A)2\023Filter by tool nameJ\022\"gmail_send_email\"'
  _globals['_LISTTOOLSRESPONSE'].fields_by_name['next_page_token']._loaded_options = None
  _globals['_LISTTOOLSRESPONSE'].fields_by_name['next_page_token']._serialized_options = b'\222AK2)Token for fetching the next page of toolsJ\036\"eyJwYWdlIjozLCJsaW1pdCI6MzB9\"'
  _globals['_LISTTOOLSRESPONSE'].fields_by_name['total_size']._loaded_options = None
  _globals['_LISTTOOLSRESPONSE'].fields_by_name['total_size']._serialized_options = b'\222A/2(Total number of tools matching the queryJ\003104'
  _globals['_LISTTOOLSRESPONSE'].fields_by_name['prev_page_token']._loaded_options = None
  _globals['_LISTTOOLSRESPONSE'].fields_by_name['prev_page_token']._serialized_options = b'\222AO2-Token for fetching the previous page of toolsJ\036\"eyJwYWdlIjoxLCJsaW1pdCI6MzB9\"'
  _globals['_LISTTOOLSRESPONSE'].fields_by_name['tool_names']._loaded_options = None
  _globals['_LISTTOOLSRESPONSE'].fields_by_name['tool_names']._serialized_options = b'\222A<2$List of tool names (if summary=true)J\024[\"gmail_send_email\"]'
  _globals['_LISTTOOLSRESPONSE'].fields_by_name['tools']._loaded_options = None
  _globals['_LISTTOOLSRESPONSE'].fields_by_name['tools']._serialized_options = b'\222A-2+List of tools (if summary=false or omitted)'
  _globals['_EXECUTETOOLREQUEST'].fields_by_name['tool_name']._loaded_options = None
  _globals['_EXECUTETOOLREQUEST'].fields_by_name['tool_name']._serialized_options = b'\222A+2\033Name of the tool to executeJ\014\"send_email\"\272H\003\310\001\001'
  _globals['_EXECUTETOOLREQUEST'].fields_by_name['identifier']._loaded_options = None
  _globals['_EXECUTETOOLREQUEST'].fields_by_name['identifier']._serialized_options = b'\222AW2=Identifier of the connected account to use for tool executionJ\026\"app_google_workspace\"'
  _globals['_EXECUTETOOLREQUEST'].fields_by_name['params']._loaded_options = None
  _globals['_EXECUTETOOLREQUEST'].fields_by_name['params']._serialized_options = b'\222A|23Free-flowing JSON parameters for the tool executionJE{\"to\": \"user@example.com\", \"subject\": \"Hello\", \"body\": \"Hello World\"}'
  _globals['_EXECUTETOOLREQUEST'].fields_by_name['connected_account_id']._loaded_options = None
  _globals['_EXECUTETOOLREQUEST'].fields_by_name['connected_account_id']._serialized_options = b'\222AA25ID of the connected account to use for tool executionJ\010\"ca_123\"\272H\004r\002\030d'
  _globals['_EXECUTETOOLREQUEST'].fields_by_name['connector']._loaded_options = None
  _globals['_EXECUTETOOLREQUEST'].fields_by_name['connector']._serialized_options = b'\222AM27Name of the connected account to use for tool executionJ\022\"Google Workspace\"\272H\004r\002\030d'
  _globals['_EXECUTETOOLREQUEST'].fields_by_name['organization_id']._loaded_options = None
  _globals['_EXECUTETOOLREQUEST'].fields_by_name['organization_id']._serialized_options = b'\222A=20ID of the organization to use for tool executionJ\t\"org_123\"\272H\004r\002\030d'
  _globals['_EXECUTETOOLREQUEST'].fields_by_name['user_id']._loaded_options = None
  _globals['_EXECUTETOOLREQUEST'].fields_by_name['user_id']._serialized_options = b'\222A62(ID of the user to use for tool executionJ\n\"user_123\"\272H\004r\002\030d'
  _globals['_EXECUTETOOLRESPONSE'].fields_by_name['data']._loaded_options = None
  _globals['_EXECUTETOOLRESPONSE'].fields_by_name['data']._serialized_options = b'\222A|23Free-flowing JSON parameters for the tool executionJE{\"to\": \"user@example.com\", \"subject\": \"Hello\", \"body\": \"Hello World\"}'
  _globals['_EXECUTETOOLRESPONSE'].fields_by_name['execution_id']._loaded_options = None
  _globals['_EXECUTETOOLRESPONSE'].fields_by_name['execution_id']._serialized_options = b'\222A72(Unique identifier for the tool executionJ\013\"123456789\"'
  _globals['_SETTOOLDEFAULTREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_SETTOOLDEFAULTREQUEST'].fields_by_name['name']._serialized_options = b'\222A/2\031Fully qualified tool nameJ\022\"gmail_send_email\"\272H\030r\023\020\001\030d2\r^[a-z0-9_].*$\310\001\001'
  _globals['_SETTOOLDEFAULTREQUEST'].fields_by_name['schema_version']._loaded_options = None
  _globals['_SETTOOLDEFAULTREQUEST'].fields_by_name['schema_version']._serialized_options = b'\222A!2\032Schema version of the toolJ\003\"1\"\272H\tr\004\020\001\030\n\310\001\001'
  _globals['_SETTOOLDEFAULTREQUEST'].fields_by_name['tool_version']._loaded_options = None
  _globals['_SETTOOLDEFAULTREQUEST'].fields_by_name['tool_version']._serialized_options = b'\222A&2\037Tool version to mark as defaultJ\003\"1\"\272H\tr\004\020\001\030\n\310\001\001'
  _globals['_SETTOOLDEFAULTRESPONSE'].fields_by_name['tool']._loaded_options = None
  _globals['_SETTOOLDEFAULTRESPONSE'].fields_by_name['tool']._serialized_options = b'\222A 2\036The tool now marked as default'
  _globals['_UPDATETOOLREQUEST'].fields_by_name['tool']._loaded_options = None
  _globals['_UPDATETOOLREQUEST'].fields_by_name['tool']._serialized_options = b'\222A\0232\021Updated tool data\272H\003\310\001\001'
  _globals['_UPDATETOOLRESPONSE'].fields_by_name['tool']._loaded_options = None
  _globals['_UPDATETOOLRESPONSE'].fields_by_name['tool']._serialized_options = b'\222A\0222\020The updated tool'
  _globals['_DELETETOOLREQUEST'].fields_by_name['id']._loaded_options = None
  _globals['_DELETETOOLREQUEST'].fields_by_name['id']._serialized_options = b'\272H\010r\006:\004tol_'
  _globals['_LISTSCOPEDTOOLSREQUEST'].fields_by_name['identifier']._loaded_options = None
  _globals['_LISTSCOPEDTOOLSREQUEST'].fields_by_name['identifier']._serialized_options = b'\222AM23Identifier of the connected account to filter toolsJ\026\"app_google_workspace\"\272H\007r\002\030d\310\001\001'
  _globals['_LISTSCOPEDTOOLSREQUEST'].fields_by_name['filter']._loaded_options = None
  _globals['_LISTSCOPEDTOOLSREQUEST'].fields_by_name['filter']._serialized_options = b'\222A,2*Filter parameters for listing scoped tools'
  _globals['_LISTSCOPEDTOOLSREQUEST'].fields_by_name['page_token']._loaded_options = None
  _globals['_LISTSCOPEDTOOLSREQUEST'].fields_by_name['page_token']._serialized_options = b'\222AN2.Token from a previous response for pagination.J\034eyJwYWdlIjoyLCJsaW1pdCI6MzB9'
  _globals['_LISTSCOPEDTOOLSRESPONSE'].fields_by_name['next_page_token']._loaded_options = None
  _globals['_LISTSCOPEDTOOLSRESPONSE'].fields_by_name['next_page_token']._serialized_options = b'\222AK2)Token for fetching the next page of toolsJ\036\"eyJwYWdlIjozLCJsaW1pdCI6MzB9\"'
  _globals['_LISTSCOPEDTOOLSRESPONSE'].fields_by_name['total_size']._loaded_options = None
  _globals['_LISTSCOPEDTOOLSRESPONSE'].fields_by_name['total_size']._serialized_options = b'\222A/2(Total number of tools matching the queryJ\003104'
  _globals['_LISTSCOPEDTOOLSRESPONSE'].fields_by_name['prev_page_token']._loaded_options = None
  _globals['_LISTSCOPEDTOOLSRESPONSE'].fields_by_name['prev_page_token']._serialized_options = b'\222AO2-Token for fetching the previous page of toolsJ\036\"eyJwYWdlIjoxLCJsaW1pdCI6MzB9\"'
  _globals['_TOOLSERVICE'].methods_by_name['CreateTool']._loaded_options = None
  _globals['_TOOLSERVICE'].methods_by_name['CreateTool']._serialized_options = b'\222Ac\n\005Tools\022\021Create a new tool\032GRegister a new tool with its schema, metadata, and other configuration.\202\265\030\002\030\004\372\322\344\223\002\t\022\007PREVIEW\202\323\344\223\002\025\"\r/api/v1/tools:\004tool'
  _globals['_TOOLSERVICE'].methods_by_name['ListTools']._loaded_options = None
  _globals['_TOOLSERVICE'].methods_by_name['ListTools']._serialized_options = b'\222At\n\005Tools\022\nList tools\032_Fetch tools by provider, identifier, and connector. If `summary=true`, returns only tool names.\202\265\030\002\030t\372\322\344\223\002\t\022\007PREVIEW\202\323\344\223\002\017\022\r/api/v1/tools'
  _globals['_TOOLSERVICE'].methods_by_name['ListScopedTools']._loaded_options = None
  _globals['_TOOLSERVICE'].methods_by_name['ListScopedTools']._serialized_options = b'\202\265\030\002\030\004\372\322\344\223\002\t\022\007PREVIEW\202\323\344\223\002\026\022\024/api/v1/tools/scoped'
  _globals['_TOOLSERVICE'].methods_by_name['SetToolDefault']._loaded_options = None
  _globals['_TOOLSERVICE'].methods_by_name['SetToolDefault']._serialized_options = b'\222A\207\001\n\005Tools\022#Marks a tool as the default version\032YSet a specific tool (by name, schema version, tool version, and provider) as the default.\202\265\030\002\030\004\372\322\344\223\002\t\022\007PREVIEW\202\323\344\223\002\036\"\031/api/v1/tools:set_default:\001*'
  _globals['_TOOLSERVICE'].methods_by_name['UpdateTool']._loaded_options = None
  _globals['_TOOLSERVICE'].methods_by_name['UpdateTool']._serialized_options = b'\222AJ\n\005Tools\022\rUpdate a tool\0322Update an existing tool by provider and tool name.\202\265\030\002\030\004\372\322\344\223\002\t\022\007PREVIEW\202\323\344\223\002\025\032\r/api/v1/tools:\004tool'
  _globals['_TOOLSERVICE'].methods_by_name['DeleteTool']._loaded_options = None
  _globals['_TOOLSERVICE'].methods_by_name['DeleteTool']._serialized_options = b'\222AZ\n\005Tools\022\rDelete a tool\032BDelete a tool by name, schema version, tool version, and provider.\202\265\030\002\030\004\372\322\344\223\002\t\022\007PREVIEW\202\323\344\223\002\024*\022/api/v1/tools/{id}'
  _globals['_TOOLSERVICE'].methods_by_name['ExecuteTool']._loaded_options = None
  _globals['_TOOLSERVICE'].methods_by_name['ExecuteTool']._serialized_options = b'\222A\215\001\n\rAppConnectors\022(Execute a tool using a connected account\032RExecute a tool using the specified connected account with the provided parameters.\202\265\030\002\030t\372\322\344\223\002\t\022\007PREVIEW\202\323\344\223\002\031\"\024/api/v1/execute_tool:\001*'
  _globals['_CREATETOOLREQUEST']._serialized_start=381
  _globals['_CREATETOOLREQUEST']._serialized_end=517
  _globals['_CREATETOOLRESPONSE']._serialized_start=519
  _globals['_CREATETOOLRESPONSE']._serialized_end=618
  _globals['_TOOL']._serialized_start=621
  _globals['_TOOL']._serialized_end=1373
  _globals['_SCOPEDTOOL']._serialized_start=1376
  _globals['_SCOPEDTOOL']._serialized_end=1587
  _globals['_LISTTOOLSREQUEST']._serialized_start=1590
  _globals['_LISTTOOLSREQUEST']._serialized_end=1844
  _globals['_FILTER']._serialized_start=1847
  _globals['_FILTER']._serialized_end=2231
  _globals['_LISTTOOLSRESPONSE']._serialized_start=2234
  _globals['_LISTTOOLSRESPONSE']._serialized_end=2773
  _globals['_EXECUTETOOLREQUEST']._serialized_start=2776
  _globals['_EXECUTETOOLREQUEST']._serialized_end=3745
  _globals['_EXECUTETOOLRESPONSE']._serialized_start=3748
  _globals['_EXECUTETOOLRESPONSE']._serialized_end=4039
  _globals['_SETTOOLDEFAULTREQUEST']._serialized_start=4042
  _globals['_SETTOOLDEFAULTREQUEST']._serialized_end=4343
  _globals['_SETTOOLDEFAULTRESPONSE']._serialized_start=4345
  _globals['_SETTOOLDEFAULTRESPONSE']._serialized_end=4451
  _globals['_UPDATETOOLREQUEST']._serialized_start=4453
  _globals['_UPDATETOOLREQUEST']._serialized_end=4547
  _globals['_UPDATETOOLRESPONSE']._serialized_start=4549
  _globals['_UPDATETOOLRESPONSE']._serialized_end=4637
  _globals['_DELETETOOLREQUEST']._serialized_start=4639
  _globals['_DELETETOOLREQUEST']._serialized_end=4687
  _globals['_LISTSCOPEDTOOLSREQUEST']._serialized_start=4690
  _globals['_LISTSCOPEDTOOLSREQUEST']._serialized_end=5091
  _globals['_LISTSCOPEDTOOLSRESPONSE']._serialized_start=5094
  _globals['_LISTSCOPEDTOOLSRESPONSE']._serialized_end=5499
  _globals['_SCOPEDTOOLFILTER']._serialized_start=5501
  _globals['_SCOPEDTOOLFILTER']._serialized_end=5623
  _globals['_TOOLSERVICE']._serialized_start=5626
  _globals['_TOOLSERVICE']._serialized_end=7332
# @@protoc_insertion_point(module_scope)




from scalekit.client import ScalekitClient
from scalekit.common.scalekit import CodeAuthenticationOptions, AuthorizationUrlOptions

__all__ = ['ScalekitClient', 'AuthorizationUrlOptions', 'CodeAuthenticationOptions']



from buf.validate import validate_pb2 as _validate_pb2
from google.api import annotations_pb2 as _annotations_pb2
from google.api import field_behavior_pb2 as _field_behavior_pb2
from google.protobuf import empty_pb2 as _empty_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf import wrappers_pb2 as _wrappers_pb2
from scalekit.v1.options import options_pb2 as _options_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class VerificationStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    VERIFICATION_STATUS_UNSPECIFIED: _ClassVar[VerificationStatus]
    PENDING: _ClassVar[VerificationStatus]
    VERIFIED: _ClassVar[VerificationStatus]
    FAILED: _ClassVar[VerificationStatus]
    AUTO_VERIFIED: _ClassVar[VerificationStatus]

class DomainType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    DOMAIN_TYPE_UNSPECIFIED: _ClassVar[DomainType]
    ALLOWED_EMAIL_DOMAIN: _ClassVar[DomainType]
    ORGANIZATION_DOMAIN: _ClassVar[DomainType]
VERIFICATION_STATUS_UNSPECIFIED: VerificationStatus
PENDING: VerificationStatus
VERIFIED: VerificationStatus
FAILED: VerificationStatus
AUTO_VERIFIED: VerificationStatus
DOMAIN_TYPE_UNSPECIFIED: DomainType
ALLOWED_EMAIL_DOMAIN: DomainType
ORGANIZATION_DOMAIN: DomainType

class CreateDomainRequest(_message.Message):
    __slots__ = ("organization_id", "external_id", "connection_id", "domain")
    ORGANIZATION_ID_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    DOMAIN_FIELD_NUMBER: _ClassVar[int]
    organization_id: str
    external_id: str
    connection_id: str
    domain: CreateDomain
    def __init__(self, organization_id: _Optional[str] = ..., external_id: _Optional[str] = ..., connection_id: _Optional[str] = ..., domain: _Optional[_Union[CreateDomain, _Mapping]] = ...) -> None: ...

class CreateDomainResponse(_message.Message):
    __slots__ = ("domain",)
    DOMAIN_FIELD_NUMBER: _ClassVar[int]
    domain: Domain
    def __init__(self, domain: _Optional[_Union[Domain, _Mapping]] = ...) -> None: ...

class CreateDomain(_message.Message):
    __slots__ = ("domain", "domain_type")
    DOMAIN_FIELD_NUMBER: _ClassVar[int]
    DOMAIN_TYPE_FIELD_NUMBER: _ClassVar[int]
    domain: str
    domain_type: DomainType
    def __init__(self, domain: _Optional[str] = ..., domain_type: _Optional[_Union[DomainType, str]] = ...) -> None: ...

class UpdateDomainRequest(_message.Message):
    __slots__ = ("organization_id", "external_id", "connection_id", "id", "domain")
    ORGANIZATION_ID_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    DOMAIN_FIELD_NUMBER: _ClassVar[int]
    organization_id: str
    external_id: str
    connection_id: str
    id: str
    domain: UpdateDomain
    def __init__(self, organization_id: _Optional[str] = ..., external_id: _Optional[str] = ..., connection_id: _Optional[str] = ..., id: _Optional[str] = ..., domain: _Optional[_Union[UpdateDomain, _Mapping]] = ...) -> None: ...

class UpdateDomain(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class UpdateDomainResponse(_message.Message):
    __slots__ = ("domain",)
    DOMAIN_FIELD_NUMBER: _ClassVar[int]
    domain: Domain
    def __init__(self, domain: _Optional[_Union[Domain, _Mapping]] = ...) -> None: ...

class GetDomainRequest(_message.Message):
    __slots__ = ("organization_id", "external_id", "id")
    ORGANIZATION_ID_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    organization_id: str
    external_id: str
    id: str
    def __init__(self, organization_id: _Optional[str] = ..., external_id: _Optional[str] = ..., id: _Optional[str] = ...) -> None: ...

class GetDomainResponse(_message.Message):
    __slots__ = ("domain",)
    DOMAIN_FIELD_NUMBER: _ClassVar[int]
    domain: Domain
    def __init__(self, domain: _Optional[_Union[Domain, _Mapping]] = ...) -> None: ...

class DeleteDomainRequest(_message.Message):
    __slots__ = ("id", "organization_id", "external_id", "connection_id")
    ID_FIELD_NUMBER: _ClassVar[int]
    ORGANIZATION_ID_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    organization_id: str
    external_id: str
    connection_id: str
    def __init__(self, id: _Optional[str] = ..., organization_id: _Optional[str] = ..., external_id: _Optional[str] = ..., connection_id: _Optional[str] = ...) -> None: ...

class ListDomainRequest(_message.Message):
    __slots__ = ("organization_id", "external_id", "connection_id", "include", "page_size", "page_number", "domain_type")
    ORGANIZATION_ID_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_NUMBER_FIELD_NUMBER: _ClassVar[int]
    DOMAIN_TYPE_FIELD_NUMBER: _ClassVar[int]
    organization_id: str
    external_id: str
    connection_id: str
    include: str
    page_size: _wrappers_pb2.Int32Value
    page_number: _wrappers_pb2.Int32Value
    domain_type: DomainType
    def __init__(self, organization_id: _Optional[str] = ..., external_id: _Optional[str] = ..., connection_id: _Optional[str] = ..., include: _Optional[str] = ..., page_size: _Optional[_Union[_wrappers_pb2.Int32Value, _Mapping]] = ..., page_number: _Optional[_Union[_wrappers_pb2.Int32Value, _Mapping]] = ..., domain_type: _Optional[_Union[DomainType, str]] = ...) -> None: ...

class VerifyDomainRequest(_message.Message):
    __slots__ = ("organization_id", "external_id", "id")
    ORGANIZATION_ID_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    organization_id: str
    external_id: str
    id: str
    def __init__(self, organization_id: _Optional[str] = ..., external_id: _Optional[str] = ..., id: _Optional[str] = ...) -> None: ...

class ListDomainResponse(_message.Message):
    __slots__ = ("page_size", "page_number", "domains")
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_NUMBER_FIELD_NUMBER: _ClassVar[int]
    DOMAINS_FIELD_NUMBER: _ClassVar[int]
    page_size: int
    page_number: int
    domains: _containers.RepeatedCompositeFieldContainer[Domain]
    def __init__(self, page_size: _Optional[int] = ..., page_number: _Optional[int] = ..., domains: _Optional[_Iterable[_Union[Domain, _Mapping]]] = ...) -> None: ...

class ListAuthorizedDomainRequest(_message.Message):
    __slots__ = ("origin",)
    ORIGIN_FIELD_NUMBER: _ClassVar[int]
    origin: str
    def __init__(self, origin: _Optional[str] = ...) -> None: ...

class ListAuthorizedDomainResponse(_message.Message):
    __slots__ = ("domains",)
    DOMAINS_FIELD_NUMBER: _ClassVar[int]
    domains: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, domains: _Optional[_Iterable[str]] = ...) -> None: ...

class Domain(_message.Message):
    __slots__ = ("id", "domain", "environment_id", "organization_id", "connection_id", "txt_record_key", "txt_record_secret", "verification_status", "create_time", "update_time", "created_by", "domain_type")
    ID_FIELD_NUMBER: _ClassVar[int]
    DOMAIN_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_ID_FIELD_NUMBER: _ClassVar[int]
    ORGANIZATION_ID_FIELD_NUMBER: _ClassVar[int]
    CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    TXT_RECORD_KEY_FIELD_NUMBER: _ClassVar[int]
    TXT_RECORD_SECRET_FIELD_NUMBER: _ClassVar[int]
    VERIFICATION_STATUS_FIELD_NUMBER: _ClassVar[int]
    CREATE_TIME_FIELD_NUMBER: _ClassVar[int]
    UPDATE_TIME_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    DOMAIN_TYPE_FIELD_NUMBER: _ClassVar[int]
    id: str
    domain: str
    environment_id: str
    organization_id: str
    connection_id: str
    txt_record_key: str
    txt_record_secret: str
    verification_status: VerificationStatus
    create_time: _timestamp_pb2.Timestamp
    update_time: _timestamp_pb2.Timestamp
    created_by: str
    domain_type: DomainType
    def __init__(self, id: _Optional[str] = ..., domain: _Optional[str] = ..., environment_id: _Optional[str] = ..., organization_id: _Optional[str] = ..., connection_id: _Optional[str] = ..., txt_record_key: _Optional[str] = ..., txt_record_secret: _Optional[str] = ..., verification_status: _Optional[_Union[VerificationStatus, str]] = ..., create_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., update_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., created_by: _Optional[str] = ..., domain_type: _Optional[_Union[DomainType, str]] = ...) -> None: ...



from buf.validate import validate_pb2 as _validate_pb2
from google.api import annotations_pb2 as _annotations_pb2
from google.api import field_behavior_pb2 as _field_behavior_pb2
from google.api import visibility_pb2 as _visibility_pb2
from google.protobuf import any_pb2 as _any_pb2
from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import empty_pb2 as _empty_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from protoc_gen_openapiv2.options import annotations_pb2 as _annotations_pb2_1
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class RegionCode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    REGION_CODE_UNSPECIFIED: _ClassVar[RegionCode]
    US: _ClassVar[RegionCode]
    EU: _ClassVar[RegionCode]

class EnvironmentType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    ENVIRONMENT_TYPE_UNSPECIFIED: _ClassVar[EnvironmentType]
    PRD: _ClassVar[EnvironmentType]
    DEV: _ClassVar[EnvironmentType]

class MembershipStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    Membership_Status_UNSPECIFIED: _ClassVar[MembershipStatus]
    ACTIVE: _ClassVar[MembershipStatus]
    INACTIVE: _ClassVar[MembershipStatus]
    PENDING_INVITE: _ClassVar[MembershipStatus]
    INVITE_EXPIRED: _ClassVar[MembershipStatus]

class IdentityProviderType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    IDENTITY_PROVIDER_UNSPECIFIED: _ClassVar[IdentityProviderType]
    OKTA: _ClassVar[IdentityProviderType]
    GOOGLE: _ClassVar[IdentityProviderType]
    MICROSOFT_AD: _ClassVar[IdentityProviderType]
    AUTH0: _ClassVar[IdentityProviderType]
    ONELOGIN: _ClassVar[IdentityProviderType]
    PING_IDENTITY: _ClassVar[IdentityProviderType]
    JUMPCLOUD: _ClassVar[IdentityProviderType]
    CUSTOM: _ClassVar[IdentityProviderType]
    GITHUB: _ClassVar[IdentityProviderType]
    GITLAB: _ClassVar[IdentityProviderType]
    LINKEDIN: _ClassVar[IdentityProviderType]
    SALESFORCE: _ClassVar[IdentityProviderType]
    MICROSOFT: _ClassVar[IdentityProviderType]
    IDP_SIMULATOR: _ClassVar[IdentityProviderType]
    SCALEKIT: _ClassVar[IdentityProviderType]
    ADFS: _ClassVar[IdentityProviderType]
REGION_CODE_UNSPECIFIED: RegionCode
US: RegionCode
EU: RegionCode
ENVIRONMENT_TYPE_UNSPECIFIED: EnvironmentType
PRD: EnvironmentType
DEV: EnvironmentType
Membership_Status_UNSPECIFIED: MembershipStatus
ACTIVE: MembershipStatus
INACTIVE: MembershipStatus
PENDING_INVITE: MembershipStatus
INVITE_EXPIRED: MembershipStatus
IDENTITY_PROVIDER_UNSPECIFIED: IdentityProviderType
OKTA: IdentityProviderType
GOOGLE: IdentityProviderType
MICROSOFT_AD: IdentityProviderType
AUTH0: IdentityProviderType
ONELOGIN: IdentityProviderType
PING_IDENTITY: IdentityProviderType
JUMPCLOUD: IdentityProviderType
CUSTOM: IdentityProviderType
GITHUB: IdentityProviderType
GITLAB: IdentityProviderType
LINKEDIN: IdentityProviderType
SALESFORCE: IdentityProviderType
MICROSOFT: IdentityProviderType
IDP_SIMULATOR: IdentityProviderType
SCALEKIT: IdentityProviderType
ADFS: IdentityProviderType

class OrganizationMembership(_message.Message):
    __slots__ = ("organization_id", "join_time", "membership_status", "roles", "name", "metadata", "display_name", "invited_by", "created_at", "accepted_at", "expires_at")
    class MetadataEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    ORGANIZATION_ID_FIELD_NUMBER: _ClassVar[int]
    JOIN_TIME_FIELD_NUMBER: _ClassVar[int]
    MEMBERSHIP_STATUS_FIELD_NUMBER: _ClassVar[int]
    ROLES_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    INVITED_BY_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    ACCEPTED_AT_FIELD_NUMBER: _ClassVar[int]
    EXPIRES_AT_FIELD_NUMBER: _ClassVar[int]
    organization_id: str
    join_time: _timestamp_pb2.Timestamp
    membership_status: MembershipStatus
    roles: _containers.RepeatedCompositeFieldContainer[Role]
    name: str
    metadata: _containers.ScalarMap[str, str]
    display_name: str
    invited_by: str
    created_at: _timestamp_pb2.Timestamp
    accepted_at: _timestamp_pb2.Timestamp
    expires_at: _timestamp_pb2.Timestamp
    def __init__(self, organization_id: _Optional[str] = ..., join_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., membership_status: _Optional[_Union[MembershipStatus, str]] = ..., roles: _Optional[_Iterable[_Union[Role, _Mapping]]] = ..., name: _Optional[str] = ..., metadata: _Optional[_Mapping[str, str]] = ..., display_name: _Optional[str] = ..., invited_by: _Optional[str] = ..., created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., accepted_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., expires_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class Role(_message.Message):
    __slots__ = ("id", "name", "display_name")
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    display_name: str
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ..., display_name: _Optional[str] = ...) -> None: ...

class UserProfile(_message.Message):
    __slots__ = ("id", "first_name", "last_name", "name", "locale", "email_verified", "phone_number", "metadata", "custom_attributes")
    class MetadataEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    class CustomAttributesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    ID_FIELD_NUMBER: _ClassVar[int]
    FIRST_NAME_FIELD_NUMBER: _ClassVar[int]
    LAST_NAME_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    LOCALE_FIELD_NUMBER: _ClassVar[int]
    EMAIL_VERIFIED_FIELD_NUMBER: _ClassVar[int]
    PHONE_NUMBER_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    CUSTOM_ATTRIBUTES_FIELD_NUMBER: _ClassVar[int]
    id: str
    first_name: str
    last_name: str
    name: str
    locale: str
    email_verified: bool
    phone_number: str
    metadata: _containers.ScalarMap[str, str]
    custom_attributes: _containers.ScalarMap[str, str]
    def __init__(self, id: _Optional[str] = ..., first_name: _Optional[str] = ..., last_name: _Optional[str] = ..., name: _Optional[str] = ..., locale: _Optional[str] = ..., email_verified: bool = ..., phone_number: _Optional[str] = ..., metadata: _Optional[_Mapping[str, str]] = ..., custom_attributes: _Optional[_Mapping[str, str]] = ...) -> None: ...



from typing import Optional, Dict, Any, List
from pydantic import BaseModel, Field
from scalekit.v1.connected_accounts.connected_accounts_pb2 import (
    CreateConnectedAccount,
    AuthorizationDetails,
    OauthToken,
    StaticAuth
)
from google.protobuf import struct_pb2


class CreateConnectedAccountRequest(BaseModel):
    """Create connected account request model"""
    
    connection_name: str = Field(..., description="Connector identifier")
    identifier: str = Field(..., description="Connected account identifier")
    authorization_details: Dict[str, Any] = Field(default_factory=dict, description="Authorization details (OAuth token or static auth)")
    organization_id: Optional[str] = Field(None, description="Organization ID")
    user_id: Optional[str] = Field(None, description="User ID")

    def to_proto(self) -> CreateConnectedAccount:
        """
        Convert to protobuf CreateConnectedAccount object
        
        :returns:
            CreateConnectedAccount protobuf object
        """
        auth_details = None
        
        if self.authorization_details and "oauth_token" in self.authorization_details:
            oauth_data = self.authorization_details["oauth_token"]
            oauth_token = OauthToken(
                access_token=oauth_data.get("access_token", ""),
                refresh_token=oauth_data.get("refresh_token", ""),
                scopes=oauth_data.get("scopes", [])
            )
            auth_details = AuthorizationDetails(oauth_token=oauth_token)
            
        elif self.authorization_details and "static_auth" in self.authorization_details:
            static_data = self.authorization_details["static_auth"]
            # Convert dict to protobuf Struct
            struct_details = struct_pb2.Struct()
            struct_details.update(static_data)
            static_auth = StaticAuth(details=struct_details)
            auth_details = AuthorizationDetails(static_auth=static_auth)
        
        elif not self.authorization_details:
            # Create empty OAuth token for empty authorization details
            oauth_token = OauthToken(
                access_token="",
                refresh_token="",
                scopes=[]
            )
            auth_details = AuthorizationDetails(oauth_token=oauth_token)
        
        return CreateConnectedAccount(authorization_details=auth_details)

    class Config:
        """Pydantic configuration"""
        validate_assignment = True


from typing import Optional, Any, Dict, List, Callable
from langchain_core.tools import StructuredTool
from scalekit.tools import ToolsClient
from scalekit.v1.tools.tools_pb2 import Filter, ScopedToolFilter


class LangChain:
    def __init__(self, tools_client: ToolsClient, execute_callback: Callable):
        if not execute_callback:
            raise ValueError("execute_callback is required. LangChain must be initialized with ConnectClient's execute_tool method.")
        
        self.tools = tools_client
        self.execute_callback = execute_callback
    
    def get_tools(
        self,
        identifier: str,
        providers: Optional[List[str]] = None,
        tool_names: Optional[List[str]] = None,
        connection_names: Optional[List[str]] = None,
        page_size: Optional[int] = None,
        page_token: Optional[str] = None
    ) -> List[StructuredTool]:
        """
        Get scoped tools from Scalekit and convert them to LangChain StructuredTools
        
        :param identifier: Identifier to scope the tools list
        :param providers: List of provider names to filter by
        :param tool_names: List of tool names to filter by
        :param connection_names: List of connection names to filter by
        :param page_size: Maximum number of tools to return per page  
        :param page_token: Token from a previous response for pagination
        :returns: List of LangChain StructuredTools
        """
        if identifier is None or identifier == "":
            raise ValueError("Identifier must be provided to get tools")

        # Create ScopedToolFilter if any filter parameters are provided
        scoped_filter = None
        if providers or tool_names or connection_names:
            scoped_filter = ScopedToolFilter(
                providers=providers or [],
                tool_names=tool_names or [],
                connection_names=connection_names or []
            )

        # Call list_scoped_tools which returns (response, metadata) tuple
        result_tuple = self.tools.list_scoped_tools(identifier, scoped_filter, page_size, page_token)
        
        # Extract the response[0] (the actual ListScopedToolsResponse proto object)
        response = result_tuple[0]
        
        structured_tools = []
        for scoped_tool in response.tools:
            structured_tool = self._convert_tool_to_structured_tool(
                scoped_tool.tool,
                scoped_tool.connected_account_id
            )
            structured_tools.append(structured_tool)
            
        return structured_tools
    
    def _convert_tool_to_structured_tool(self, tool, connected_account_id: str) -> StructuredTool:
        """Convert a Scalekit Tool to LangChain StructuredTool"""
        

        definition_dict = self._struct_to_dict(tool.definition) if hasattr(tool, 'definition') and tool.definition else {}
        

        tool_name = definition_dict.get('name', getattr(tool, 'provider', 'unknown') + '_tool')
        tool_description = definition_dict.get('description', 'Scalekit tool')
        

        args_schema = definition_dict.get("input_schema", {})
        

        def _call(**arguments: Dict[str, Any]) -> str:
            try:
                # Import here to avoid circular imports
                from scalekit.connect.types import ToolInput

                
                # Call connect.execute_tool via callback (includes modifiers and enhanced handling)
                response = self.execute_callback(
                    tool_input=arguments,
                    tool_name=tool_name,
                    connected_account_id=connected_account_id
                )


                result_data = response.data if hasattr(response, 'data') else {}

                execution_id = response.execution_id if hasattr(response, 'execution_id') else None
                
                # Format the response
                result_dict = dict(result_data) if result_data else {}
                if execution_id:
                    result_dict['execution_id'] = execution_id
                
                return str(result_dict) if result_dict else f"Tool {tool_name} executed successfully"
                
            except Exception as e:
                return f"Error executing tool {tool_name}: {str(e)}"
        
        # Sync wrapper
        def call_tool_sync(**arguments: Dict[str, Any]) -> str:
            return _call(**arguments)
        
        # Async wrapper
        async def call_tool_async(**arguments: Dict[str, Any]) -> str:
            return _call(**arguments)

        # TODO add metadata to the tool if available
        return StructuredTool(
            name=tool_name,
            description=tool_description,
            args_schema=args_schema,
            func=call_tool_sync,
            coroutine=call_tool_async,
        )
    
    def _struct_to_dict(self, struct) -> Dict[str, Any]:
        """Convert protobuf Struct to Python dict"""
        from google.protobuf.json_format import MessageToDict
        return MessageToDict(struct)


from typing import Optional, Dict, Union

from scalekit.core import CoreClient
from scalekit.v1.auth.passwordless_pb2 import *
from scalekit.v1.auth.passwordless_pb2_grpc import PasswordlessServiceStub



# String to enum mapping for template types
_TEMPLATE_TYPE_MAP = {
    "UNSPECIFIED": TemplateType.UNSPECIFIED,
    "SIGNIN": TemplateType.SIGNIN,
    "SIGNUP": TemplateType.SIGNUP,
}

def _convert_template_type(template: Union[str, TemplateType, None]) -> Optional[TemplateType]:
    """Convert string or enum to TemplateType enum"""
    if template is None:
        return None
    if isinstance(template, str):
        template_upper = template.upper()
        if template_upper in _TEMPLATE_TYPE_MAP:
            return _TEMPLATE_TYPE_MAP[template_upper]
        else:
            raise ValueError(f"Invalid template type: {template}. Valid options are: {list(_TEMPLATE_TYPE_MAP.keys())}")
    return template


class PasswordlessClient:
    """Class definition for Passwordless Client"""

    def __init__(self, core_client: CoreClient):
        """
        Initializer for Passwordless Client

        :param core_client    : CoreClient Object
        :type                 : ``` obj ```
        :returns
            None
        """
        self.core_client = core_client
        self.passwordless_service = PasswordlessServiceStub(
            self.core_client.grpc_secure_channel
        )

    def send_passwordless_email(
        self,
        email: str,
        template: Optional[Union[str, TemplateType]] = None,
        magiclink_auth_uri: Optional[str] = None,
        state: Optional[str] = None,
        expires_in: Optional[int] = None,
        template_variables: Optional[Dict[str, str]] = None
    ) -> SendPasswordlessResponse:
        """
        Method to send passwordless authentication email

        :param email              : Email address to send the passwordless authentication to
        :type                     : ``` str ```
        :param template           : Template type ("SIGNIN", "SIGNUP", or "UNSPECIFIED")
        :type                     : ``` str or TemplateType ```
        :param magiclink_auth_uri : Callback URL for magic link authentication
        :type                     : ``` str ```
        :param state              : Custom state parameter to maintain application state
        :type                     : ``` str ```
        :param expires_in         : Expiration time in seconds (default: 300)
        :type                     : ``` int ```
        :param template_variables : Key-value pairs for email template personalization
        :type                     : ``` dict ```

        :returns:
            Send Passwordless Response
        """
        request = SendPasswordlessRequest(email=email)
        
        # Convert template string to enum if needed
        template_enum = _convert_template_type(template)
        if template_enum is not None:
            request.template = template_enum
            
        if magiclink_auth_uri is not None:
            request.magiclink_auth_uri = magiclink_auth_uri
        if state is not None:
            request.state = state
        if expires_in is not None:
            request.expires_in = expires_in
        if template_variables is not None:
            request.template_variables.update(template_variables)
        
        return self.core_client.grpc_exec(
            self.passwordless_service.SendPasswordlessEmail.with_call,
            request,
        )

    def verify_passwordless_email(
        self,
        code: Optional[str] = None,
        link_token: Optional[str] = None,
        auth_request_id: Optional[str] = None
    ) -> VerifyPasswordLessResponse:
        """
        Method to verify passwordless authentication

        :param code            : The OTP code received via email
        :type                  : ``` str ```
        :param link_token      : The token from the magic link URL
        :type                  : ``` str ```
        :param auth_request_id : The authentication request ID from send request
        :type                  : ``` str ```

        :returns:
            Verify Passwordless Response
        """
        request = VerifyPasswordLessRequest()
        
        if code is not None:
            request.code = code
        elif link_token is not None:
            request.link_token = link_token
            
        if auth_request_id is not None:
            request.auth_request_id = auth_request_id
        
        return self.core_client.grpc_exec(
            self.passwordless_service.VerifyPasswordlessEmail.with_call,
            request,
        )

    def resend_passwordless_email(
        self,
        auth_request_id: str
    ) -> SendPasswordlessResponse:
        """
        Method to resend passwordless authentication email

        :param auth_request_id : The authentication request ID from original send request
        :type                  : ``` str ```

        :returns:
            Send Passwordless Response
        """
        return self.core_client.grpc_exec(
            self.passwordless_service.ResendPasswordlessEmail.with_call,
            ResendPasswordlessRequest(auth_request_id=auth_request_id),
        ) 



from faker import Faker

from basetest import BaseTest
from scalekit.v1.organizations.organizations_pb2 import CreateOrganization


class TestDomain(BaseTest):
    """ Class definition for Test Domain Class """
    def setUp(self):
        """ """
        self.org_id = None

    def test_create_domain(self):
        """ Method to test create domain """
        organization = CreateOrganization(display_name=Faker().company(), external_id=Faker().uuid4())
        org_response = self.scalekit_client.organization.create_organization(organization=organization)
        self.org_id = org_response[0].organization.id

        domain_name = Faker().domain_name()
        response = self.scalekit_client.domain.create_domain(organization_id=self.org_id, domain_name=domain_name)
        self.assertEqual(response[1].code().name, "OK")
        self.assertTrue(response[0] is not None)
        self.assertEqual(response[0].domain.domain, domain_name)
        self.assertEqual(response[0].domain.organization_id, self.org_id)

    def test_get_domain(self):
        """ Method to test get domain """
        organization = CreateOrganization(display_name=Faker().company(), external_id=Faker().uuid4())
        org_response = self.scalekit_client.organization.create_organization(organization=organization)
        self.org_id = org_response[0].organization.id

        domain_name = Faker().domain_name()
        create_domain_response = self.scalekit_client.domain.create_domain(
            organization_id=self.org_id, domain_name=domain_name)
        domain_id = create_domain_response[0].domain.id

        response = self.scalekit_client.domain.get_domain(organization_id=self.org_id, domain_id=domain_id)
        self.assertEqual(response[1].code().name, "OK")
        self.assertTrue(response[0] is not None)
        self.assertEqual(response[0].domain.id, domain_id)
        self.assertEqual(response[0].domain.domain, domain_name)
        self.assertEqual(response[0].domain.organization_id, self.org_id)

    def test_list_domains(self):
        """ Method to test list domains """
        organization = CreateOrganization(display_name=Faker().company(), external_id=Faker().uuid4())
        org_response = self.scalekit_client.organization.create_organization(organization=organization)
        self.org_id = org_response[0].organization.id

        domain_name = Faker().domain_name()
        create_domain_response = self.scalekit_client.domain.create_domain(
            organization_id=self.org_id, domain_name=domain_name)
        self.assertEqual(create_domain_response[1].code().name, "OK")

        response = self.scalekit_client.domain.list_domains(organization_id=self.org_id)
        self.assertEqual(response[1].code().name, "OK")
        self.assertTrue(response[0] is not None)
        self.assertTrue(len(response[0].domains) > 0)
        self.assertEqual(response[0].domains[0].id, create_domain_response[0].domain.id)
        self.assertEqual(response[0].domains[0].domain, domain_name)
        self.assertEqual(response[0].domains[0].organization_id, self.org_id)

    def tearDown(self):
        """ Method to clean up """
        if self.org_id:
            self.scalekit_client.organization.delete_organization(organization_id=self.org_id)



from setuptools import setup, find_packages

setup(
    name="a-beta-scalekit",
    version="4.0.2",
    packages=find_packages(),
    install_requires=[
        "grpcio>=1.64.1",
        "protobuf<6.0.0,>=5.26.1",
        "google>=3.0",
        "requests>=2.32.3",
        "PyJWT>=2.8,<2.10",
        "cffi>=1.15.1",
        "cryptography==44.0.1",
        "setuptools>=78.1.1",
        "grpcio-status>=1.64,<1.67",
        "protoc-gen-openapiv2>=0.0.1",
        "googleapis-common-protos>=1.56.1,<1.66.0",
        "deprecation>=2.1.0",
        "python-dotenv~=1.0.1",
        "Faker~=25.8.0",
        "pydantic~=2.10.6",
        "langchain-core>=0.3.36,<0.4",
    ],
    url="https://github.com/scalekit-inc/scalekit-sdk-python",
    license="MIT",
    author="Team Scalekit",
    author_email="support@scalekit.com",
    description="Scalekit official Python SDK",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: scalekit/v1/user_attributes/user_attributes.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from scalekit.v1.commons import commons_pb2 as scalekit_dot_v1_dot_commons_dot_commons__pb2
from scalekit.v1.options import options_pb2 as scalekit_dot_v1_dot_options_dot_options__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1scalekit/v1/user_attributes/user_attributes.proto\x12\x1bscalekit.v1.user_attributes\x1a\x1b\x62uf/validate/validate.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/protobuf/any.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a!scalekit/v1/commons/commons.proto\x1a!scalekit/v1/options/options.proto\"\xc3\x04\n\rUserAttribute\x12\x36\n\x03key\x18\x01 \x01(\tB$\xbaH!r\x1f\x10\x03\x18\x19\x32\x19^[a-zA-Z][a-zA-Z0-9\\-_]+$R\x03key\x12\x18\n\x07\x65nabled\x18\x02 \x01(\x08R\x07\x65nabled\x12\x1a\n\x08required\x18\x03 \x01(\x08R\x08required\x12 \n\x05label\x18\x04 \x01(\tB\n\xbaH\x07r\x05\x10\x03\x18\xfa\x01R\x05label\x12Y\n\x08\x64\x61tatype\x18\x05 \x01(\x0e\x32\x32.scalekit.v1.user_attributes.UserAttributeDatatypeB\t\xbaH\x06\x82\x01\x03\"\x01\x00R\x08\x64\x61tatype\x12S\n\x08\x63\x61tegory\x18\x06 \x01(\x0e\x32\x32.scalekit.v1.user_attributes.UserAttributeCategoryB\x03\xe0\x41\x03R\x08\x63\x61tegory\x12^\n\x11sso_addition_info\x18\x07 \x01(\x0b\x32\x30.scalekit.v1.user_attributes.SSOUserAdditionInfoH\x00R\x0fssoAdditionInfo\x12\x7f\n\x1e\x64irectory_user_additional_info\x18\x08 \x01(\x0b\x32\x38.scalekit.v1.user_attributes.DirectoryUserAdditionalInfoH\x00R\x1b\x64irectoryUserAdditionalInfoB\x11\n\x0f\x61\x64\x64itional_info\"\xc5\x01\n\x13SSOUserAdditionInfo\x12V\n\x14\x64\x65\x66\x61ult_saml_mapping\x18\x01 \x01(\tB$\xbaH!r\x1f\x10\x00\x18\x19\x32\x19^[a-zA-Z][a-zA-Z0-9\\-_]+$R\x12\x64\x65\x66\x61ultSamlMapping\x12V\n\x14\x64\x65\x66\x61ult_oidc_mapping\x18\x02 \x01(\tB$\xbaH!r\x1f\x10\x00\x18\x19\x32\x19^[a-zA-Z][a-zA-Z0-9\\-_]+$R\x12\x64\x65\x66\x61ultOidcMapping\"\x1d\n\x1b\x44irectoryUserAdditionalInfo\"w\n\x1a\x43reateUserAttributeRequest\x12Y\n\x0euser_attribute\x18\x01 \x01(\x0b\x32*.scalekit.v1.user_attributes.UserAttributeB\x06\xbaH\x03\xc8\x01\x01R\ruserAttribute\"p\n\x1b\x43reateUserAttributeResponse\x12Q\n\x0euser_attribute\x18\x01 \x01(\x0b\x32*.scalekit.v1.user_attributes.UserAttributeR\ruserAttribute\"q\n\x1aListUserAttributesResponse\x12S\n\x0fuser_attributes\x18\x01 \x03(\x0b\x32*.scalekit.v1.user_attributes.UserAttributeR\x0euserAttributes\"\x95\x01\n\x1aUpdateUserAttributeRequest\x12\x1c\n\x03key\x18\x01 \x01(\tB\n\xbaH\x07r\x05\x10\x03\x18\xfa\x01R\x03key\x12Y\n\x0euser_attribute\x18\x02 \x01(\x0b\x32*.scalekit.v1.user_attributes.UserAttributeB\x06\xbaH\x03\xc8\x01\x01R\ruserAttribute\"p\n\x1bUpdateUserAttributeResponse\x12Q\n\x0euser_attribute\x18\x01 \x01(\x0b\x32*.scalekit.v1.user_attributes.UserAttributeR\ruserAttribute\":\n\x1a\x44\x65leteUserAttributeRequest\x12\x1c\n\x03key\x18\x01 \x01(\tB\n\xbaH\x07r\x05\x10\x03\x18\xfa\x01R\x03key*j\n\x15UserAttributeDatatype\x12\x18\n\x14UNSPECIFIED_DATATYPE\x10\x00\x12\n\n\x06STRING\x10\x01\x12\n\n\x06NUMBER\x10\x02\x12\x08\n\x04\x42OOL\x10\x03\x12\t\n\x05\x41RRAY\x10\x04\x12\n\n\x06OBJECT\x10\x05*K\n\x15UserAttributeCategory\x12\x18\n\x14UNSPECIFIED_CATEGORY\x10\x00\x12\x0c\n\x08STANDARD\x10\x01\x12\n\n\x06\x43USTOM\x10\x02\x32\x94\r\n\x14UserAttributeService\x12\xf9\x01\n\x16\x43reateSsoUserAttribute\x12\x37.scalekit.v1.user_attributes.CreateUserAttributeRequest\x1a\x38.scalekit.v1.user_attributes.CreateUserAttributeResponse\"l\x82\xb5\x18\x02\x18t\x82\xd3\xe4\x93\x02`\"\x1b/api/v1/sso-user-attributes:\x0euser_attributeZ1\"\x1f/api/v1/user-profile-attributes:\x0euser_attribute\x12\xb6\x01\n\x15ListSsoUserAttributes\x12\x16.google.protobuf.Empty\x1a\x37.scalekit.v1.user_attributes.ListUserAttributesResponse\"L\x82\xb5\x18\x02\x18t\x82\xd3\xe4\x93\x02@\x12\x1b/api/v1/sso-user-attributesZ!\x12\x1f/api/v1/user-profile-attributes\x12\x85\x02\n\x16UpdateSsoUserAttribute\x12\x37.scalekit.v1.user_attributes.UpdateUserAttributeRequest\x1a\x38.scalekit.v1.user_attributes.UpdateUserAttributeResponse\"x\x82\xb5\x18\x02\x18t\x82\xd3\xe4\x93\x02l2!/api/v1/sso-user-attributes/{key}:\x0euser_attributeZ72%/api/v1/user-profile-attributes/{key}:\x0euser_attribute\x12\xc3\x01\n\x16\x44\x65leteSsoUserAttribute\x12\x37.scalekit.v1.user_attributes.DeleteUserAttributeRequest\x1a\x16.google.protobuf.Empty\"X\x82\xb5\x18\x02\x18t\x82\xd3\xe4\x93\x02L*!/api/v1/sso-user-attributes/{key}Z\'*%/api/v1/user-profile-attributes/{key}\x12\xd2\x01\n\x1c\x43reateDirectoryUserAttribute\x12\x37.scalekit.v1.user_attributes.CreateUserAttributeRequest\x1a\x38.scalekit.v1.user_attributes.CreateUserAttributeResponse\"?\x82\xb5\x18\x02\x18t\x82\xd3\xe4\x93\x02\x33\"!/api/v1/directory-user-attributes:\x0euser_attribute\x12\x9f\x01\n\x1bListDirectoryUserAttributes\x12\x16.google.protobuf.Empty\x1a\x37.scalekit.v1.user_attributes.ListUserAttributesResponse\"/\x82\xb5\x18\x02\x18t\x82\xd3\xe4\x93\x02#\x12!/api/v1/directory-user-attributes\x12\xd8\x01\n\x1cUpdateDirectoryUserAttribute\x12\x37.scalekit.v1.user_attributes.UpdateUserAttributeRequest\x1a\x38.scalekit.v1.user_attributes.UpdateUserAttributeResponse\"E\x82\xb5\x18\x02\x18t\x82\xd3\xe4\x93\x02\x39\x32\'/api/v1/directory-user-attributes/{key}:\x0euser_attribute\x12\xa6\x01\n\x1c\x44\x65leteDirectoryUserAttribute\x12\x37.scalekit.v1.user_attributes.DeleteUserAttributeRequest\x1a\x16.google.protobuf.Empty\"5\x82\xb5\x18\x02\x18t\x82\xd3\xe4\x93\x02)*\'/api/v1/directory-user-attributes/{key}B;Z9github.com/scalekit-inc/scalekit/pkg/grpc/user_attributesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scalekit.v1.user_attributes.user_attributes_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z9github.com/scalekit-inc/scalekit/pkg/grpc/user_attributes'
  _globals['_USERATTRIBUTE'].fields_by_name['key']._loaded_options = None
  _globals['_USERATTRIBUTE'].fields_by_name['key']._serialized_options = b'\272H!r\037\020\003\030\0312\031^[a-zA-Z][a-zA-Z0-9\\-_]+$'
  _globals['_USERATTRIBUTE'].fields_by_name['label']._loaded_options = None
  _globals['_USERATTRIBUTE'].fields_by_name['label']._serialized_options = b'\272H\007r\005\020\003\030\372\001'
  _globals['_USERATTRIBUTE'].fields_by_name['datatype']._loaded_options = None
  _globals['_USERATTRIBUTE'].fields_by_name['datatype']._serialized_options = b'\272H\006\202\001\003\"\001\000'
  _globals['_USERATTRIBUTE'].fields_by_name['category']._loaded_options = None
  _globals['_USERATTRIBUTE'].fields_by_name['category']._serialized_options = b'\340A\003'
  _globals['_SSOUSERADDITIONINFO'].fields_by_name['default_saml_mapping']._loaded_options = None
  _globals['_SSOUSERADDITIONINFO'].fields_by_name['default_saml_mapping']._serialized_options = b'\272H!r\037\020\000\030\0312\031^[a-zA-Z][a-zA-Z0-9\\-_]+$'
  _globals['_SSOUSERADDITIONINFO'].fields_by_name['default_oidc_mapping']._loaded_options = None
  _globals['_SSOUSERADDITIONINFO'].fields_by_name['default_oidc_mapping']._serialized_options = b'\272H!r\037\020\000\030\0312\031^[a-zA-Z][a-zA-Z0-9\\-_]+$'
  _globals['_CREATEUSERATTRIBUTEREQUEST'].fields_by_name['user_attribute']._loaded_options = None
  _globals['_CREATEUSERATTRIBUTEREQUEST'].fields_by_name['user_attribute']._serialized_options = b'\272H\003\310\001\001'
  _globals['_UPDATEUSERATTRIBUTEREQUEST'].fields_by_name['key']._loaded_options = None
  _globals['_UPDATEUSERATTRIBUTEREQUEST'].fields_by_name['key']._serialized_options = b'\272H\007r\005\020\003\030\372\001'
  _globals['_UPDATEUSERATTRIBUTEREQUEST'].fields_by_name['user_attribute']._loaded_options = None
  _globals['_UPDATEUSERATTRIBUTEREQUEST'].fields_by_name['user_attribute']._serialized_options = b'\272H\003\310\001\001'
  _globals['_DELETEUSERATTRIBUTEREQUEST'].fields_by_name['key']._loaded_options = None
  _globals['_DELETEUSERATTRIBUTEREQUEST'].fields_by_name['key']._serialized_options = b'\272H\007r\005\020\003\030\372\001'
  _globals['_USERATTRIBUTESERVICE'].methods_by_name['CreateSsoUserAttribute']._loaded_options = None
  _globals['_USERATTRIBUTESERVICE'].methods_by_name['CreateSsoUserAttribute']._serialized_options = b'\202\265\030\002\030t\202\323\344\223\002`\"\033/api/v1/sso-user-attributes:\016user_attributeZ1\"\037/api/v1/user-profile-attributes:\016user_attribute'
  _globals['_USERATTRIBUTESERVICE'].methods_by_name['ListSsoUserAttributes']._loaded_options = None
  _globals['_USERATTRIBUTESERVICE'].methods_by_name['ListSsoUserAttributes']._serialized_options = b'\202\265\030\002\030t\202\323\344\223\002@\022\033/api/v1/sso-user-attributesZ!\022\037/api/v1/user-profile-attributes'
  _globals['_USERATTRIBUTESERVICE'].methods_by_name['UpdateSsoUserAttribute']._loaded_options = None
  _globals['_USERATTRIBUTESERVICE'].methods_by_name['UpdateSsoUserAttribute']._serialized_options = b'\202\265\030\002\030t\202\323\344\223\002l2!/api/v1/sso-user-attributes/{key}:\016user_attributeZ72%/api/v1/user-profile-attributes/{key}:\016user_attribute'
  _globals['_USERATTRIBUTESERVICE'].methods_by_name['DeleteSsoUserAttribute']._loaded_options = None
  _globals['_USERATTRIBUTESERVICE'].methods_by_name['DeleteSsoUserAttribute']._serialized_options = b'\202\265\030\002\030t\202\323\344\223\002L*!/api/v1/sso-user-attributes/{key}Z\'*%/api/v1/user-profile-attributes/{key}'
  _globals['_USERATTRIBUTESERVICE'].methods_by_name['CreateDirectoryUserAttribute']._loaded_options = None
  _globals['_USERATTRIBUTESERVICE'].methods_by_name['CreateDirectoryUserAttribute']._serialized_options = b'\202\265\030\002\030t\202\323\344\223\0023\"!/api/v1/directory-user-attributes:\016user_attribute'
  _globals['_USERATTRIBUTESERVICE'].methods_by_name['ListDirectoryUserAttributes']._loaded_options = None
  _globals['_USERATTRIBUTESERVICE'].methods_by_name['ListDirectoryUserAttributes']._serialized_options = b'\202\265\030\002\030t\202\323\344\223\002#\022!/api/v1/directory-user-attributes'
  _globals['_USERATTRIBUTESERVICE'].methods_by_name['UpdateDirectoryUserAttribute']._loaded_options = None
  _globals['_USERATTRIBUTESERVICE'].methods_by_name['UpdateDirectoryUserAttribute']._serialized_options = b'\202\265\030\002\030t\202\323\344\223\00292\'/api/v1/directory-user-attributes/{key}:\016user_attribute'
  _globals['_USERATTRIBUTESERVICE'].methods_by_name['DeleteDirectoryUserAttribute']._loaded_options = None
  _globals['_USERATTRIBUTESERVICE'].methods_by_name['DeleteDirectoryUserAttribute']._serialized_options = b'\202\265\030\002\030t\202\323\344\223\002)*\'/api/v1/directory-user-attributes/{key}'
  _globals['_USERATTRIBUTEDATATYPE']._serialized_start=1884
  _globals['_USERATTRIBUTEDATATYPE']._serialized_end=1990
  _globals['_USERATTRIBUTECATEGORY']._serialized_start=1992
  _globals['_USERATTRIBUTECATEGORY']._serialized_end=2067
  _globals['_USERATTRIBUTE']._serialized_start=396
  _globals['_USERATTRIBUTE']._serialized_end=975
  _globals['_SSOUSERADDITIONINFO']._serialized_start=978
  _globals['_SSOUSERADDITIONINFO']._serialized_end=1175
  _globals['_DIRECTORYUSERADDITIONALINFO']._serialized_start=1177
  _globals['_DIRECTORYUSERADDITIONALINFO']._serialized_end=1206
  _globals['_CREATEUSERATTRIBUTEREQUEST']._serialized_start=1208
  _globals['_CREATEUSERATTRIBUTEREQUEST']._serialized_end=1327
  _globals['_CREATEUSERATTRIBUTERESPONSE']._serialized_start=1329
  _globals['_CREATEUSERATTRIBUTERESPONSE']._serialized_end=1441
  _globals['_LISTUSERATTRIBUTESRESPONSE']._serialized_start=1443
  _globals['_LISTUSERATTRIBUTESRESPONSE']._serialized_end=1556
  _globals['_UPDATEUSERATTRIBUTEREQUEST']._serialized_start=1559
  _globals['_UPDATEUSERATTRIBUTEREQUEST']._serialized_end=1708
  _globals['_UPDATEUSERATTRIBUTERESPONSE']._serialized_start=1710
  _globals['_UPDATEUSERATTRIBUTERESPONSE']._serialized_end=1822
  _globals['_DELETEUSERATTRIBUTEREQUEST']._serialized_start=1824
  _globals['_DELETEUSERATTRIBUTEREQUEST']._serialized_end=1882
  _globals['_USERATTRIBUTESERVICE']._serialized_start=2070
  _globals['_USERATTRIBUTESERVICE']._serialized_end=3754
# @@protoc_insertion_point(module_scope)



from buf.validate import validate_pb2 as _validate_pb2
from google.api import annotations_pb2 as _annotations_pb2
from google.api import field_behavior_pb2 as _field_behavior_pb2
from google.protobuf import any_pb2 as _any_pb2
from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import empty_pb2 as _empty_pb2
from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from scalekit.v1.commons import commons_pb2 as _commons_pb2
from scalekit.v1.options import options_pb2 as _options_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class UserAttributeDatatype(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    UNSPECIFIED_DATATYPE: _ClassVar[UserAttributeDatatype]
    STRING: _ClassVar[UserAttributeDatatype]
    NUMBER: _ClassVar[UserAttributeDatatype]
    BOOL: _ClassVar[UserAttributeDatatype]
    ARRAY: _ClassVar[UserAttributeDatatype]
    OBJECT: _ClassVar[UserAttributeDatatype]

class UserAttributeCategory(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    UNSPECIFIED_CATEGORY: _ClassVar[UserAttributeCategory]
    STANDARD: _ClassVar[UserAttributeCategory]
    CUSTOM: _ClassVar[UserAttributeCategory]
UNSPECIFIED_DATATYPE: UserAttributeDatatype
STRING: UserAttributeDatatype
NUMBER: UserAttributeDatatype
BOOL: UserAttributeDatatype
ARRAY: UserAttributeDatatype
OBJECT: UserAttributeDatatype
UNSPECIFIED_CATEGORY: UserAttributeCategory
STANDARD: UserAttributeCategory
CUSTOM: UserAttributeCategory

class UserAttribute(_message.Message):
    __slots__ = ("key", "enabled", "required", "label", "datatype", "category", "sso_addition_info", "directory_user_additional_info")
    KEY_FIELD_NUMBER: _ClassVar[int]
    ENABLED_FIELD_NUMBER: _ClassVar[int]
    REQUIRED_FIELD_NUMBER: _ClassVar[int]
    LABEL_FIELD_NUMBER: _ClassVar[int]
    DATATYPE_FIELD_NUMBER: _ClassVar[int]
    CATEGORY_FIELD_NUMBER: _ClassVar[int]
    SSO_ADDITION_INFO_FIELD_NUMBER: _ClassVar[int]
    DIRECTORY_USER_ADDITIONAL_INFO_FIELD_NUMBER: _ClassVar[int]
    key: str
    enabled: bool
    required: bool
    label: str
    datatype: UserAttributeDatatype
    category: UserAttributeCategory
    sso_addition_info: SSOUserAdditionInfo
    directory_user_additional_info: DirectoryUserAdditionalInfo
    def __init__(self, key: _Optional[str] = ..., enabled: bool = ..., required: bool = ..., label: _Optional[str] = ..., datatype: _Optional[_Union[UserAttributeDatatype, str]] = ..., category: _Optional[_Union[UserAttributeCategory, str]] = ..., sso_addition_info: _Optional[_Union[SSOUserAdditionInfo, _Mapping]] = ..., directory_user_additional_info: _Optional[_Union[DirectoryUserAdditionalInfo, _Mapping]] = ...) -> None: ...

class SSOUserAdditionInfo(_message.Message):
    __slots__ = ("default_saml_mapping", "default_oidc_mapping")
    DEFAULT_SAML_MAPPING_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_OIDC_MAPPING_FIELD_NUMBER: _ClassVar[int]
    default_saml_mapping: str
    default_oidc_mapping: str
    def __init__(self, default_saml_mapping: _Optional[str] = ..., default_oidc_mapping: _Optional[str] = ...) -> None: ...

class DirectoryUserAdditionalInfo(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class CreateUserAttributeRequest(_message.Message):
    __slots__ = ("user_attribute",)
    USER_ATTRIBUTE_FIELD_NUMBER: _ClassVar[int]
    user_attribute: UserAttribute
    def __init__(self, user_attribute: _Optional[_Union[UserAttribute, _Mapping]] = ...) -> None: ...

class CreateUserAttributeResponse(_message.Message):
    __slots__ = ("user_attribute",)
    USER_ATTRIBUTE_FIELD_NUMBER: _ClassVar[int]
    user_attribute: UserAttribute
    def __init__(self, user_attribute: _Optional[_Union[UserAttribute, _Mapping]] = ...) -> None: ...

class ListUserAttributesResponse(_message.Message):
    __slots__ = ("user_attributes",)
    USER_ATTRIBUTES_FIELD_NUMBER: _ClassVar[int]
    user_attributes: _containers.RepeatedCompositeFieldContainer[UserAttribute]
    def __init__(self, user_attributes: _Optional[_Iterable[_Union[UserAttribute, _Mapping]]] = ...) -> None: ...

class UpdateUserAttributeRequest(_message.Message):
    __slots__ = ("key", "user_attribute")
    KEY_FIELD_NUMBER: _ClassVar[int]
    USER_ATTRIBUTE_FIELD_NUMBER: _ClassVar[int]
    key: str
    user_attribute: UserAttribute
    def __init__(self, key: _Optional[str] = ..., user_attribute: _Optional[_Union[UserAttribute, _Mapping]] = ...) -> None: ...

class UpdateUserAttributeResponse(_message.Message):
    __slots__ = ("user_attribute",)
    USER_ATTRIBUTE_FIELD_NUMBER: _ClassVar[int]
    user_attribute: UserAttribute
    def __init__(self, user_attribute: _Optional[_Union[UserAttribute, _Mapping]] = ...) -> None: ...

class DeleteUserAttributeRequest(_message.Message):
    __slots__ = ("key",)
    KEY_FIELD_NUMBER: _ClassVar[int]
    key: str
    def __init__(self, key: _Optional[str] = ...) -> None: ...



from google.protobuf import descriptor_pb2 as _descriptor_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor
FIELD_FIELD_NUMBER: _ClassVar[int]
field: _descriptor.FieldDescriptor

class FieldConstraints(_message.Message):
    __slots__ = ("cel",)
    CEL_FIELD_NUMBER: _ClassVar[int]
    cel: _containers.RepeatedCompositeFieldContainer[Constraint]
    def __init__(self, cel: _Optional[_Iterable[_Union[Constraint, _Mapping]]] = ...) -> None: ...

class Constraint(_message.Message):
    __slots__ = ("id", "message", "expression")
    ID_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    EXPRESSION_FIELD_NUMBER: _ClassVar[int]
    id: str
    message: str
    expression: str
    def __init__(self, id: _Optional[str] = ..., message: _Optional[str] = ..., expression: _Optional[str] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: buf/validate/priv/private.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1f\x62uf/validate/priv/private.proto\x12\x11\x62uf.validate.priv\x1a google/protobuf/descriptor.proto\"C\n\x10\x46ieldConstraints\x12/\n\x03\x63\x65l\x18\x01 \x03(\x0b\x32\x1d.buf.validate.priv.ConstraintR\x03\x63\x65l\"V\n\nConstraint\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\x12\x1e\n\nexpression\x18\x03 \x01(\tR\nexpression:\\\n\x05\x66ield\x12\x1d.google.protobuf.FieldOptions\x18\x88\t \x01(\x0b\x32#.buf.validate.priv.FieldConstraintsR\x05\x66ield\x88\x01\x01\x42w\n\x17\x62uild.buf.validate.privB\x0cPrivateProtoP\x01ZLbuf.build/gen/go/bufbuild/protovalidate/protocolbuffers/go/buf/validate/privb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'buf.validate.priv.private_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\027build.buf.validate.privB\014PrivateProtoP\001ZLbuf.build/gen/go/bufbuild/protovalidate/protocolbuffers/go/buf/validate/priv'
  _globals['_FIELDCONSTRAINTS']._serialized_start=88
  _globals['_FIELDCONSTRAINTS']._serialized_end=155
  _globals['_CONSTRAINT']._serialized_start=157
  _globals['_CONSTRAINT']._serialized_end=243
# @@protoc_insertion_point(module_scope)
